@@ -1,27 +1,15 @@
 /*  This file is part of Kleopatra, the KDE keymanager
-    Copyright (c) 2016 Klarälvdalens Datakonsult AB
+    SPDX-FileCopyrightText: 2016 Klarälvdalens Datakonsult AB
 
-    Kleopatra is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    Kleopatra is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+    SPDX-License-Identifier: GPL-2.0-or-later
 */
 
-#ifndef LIBKLEO_DEFAULTKEYGENERATIONJOB_H
-#define LIBKLEO_DEFAULTKEYGENERATIONJOB_H
+#pragma once
 
 #include <qgpgme/job.h>
 
-#include <kleo_export.h>
+#include "kleo_export.h"
+#include <memory>
 
 namespace GpgME {
 class KeyGenerationResult;
@@ -67,10 +55,9 @@ protected:
     bool eventFilter(QObject *watched, QEvent *event) override;
 
 private:
-    class Private;
-    Private * const d;
+    class DefaultKeyGenerationJobPrivate;
+    std::unique_ptr<DefaultKeyGenerationJobPrivate> const d;
 };
 
 }
 
-#endif
