@@ -2,32 +2,9 @@
     kleo/enum.cpp
 
     This file is part of libkleopatra, the KDE keymanagement library
-    Copyright (c) 2004 Klarälvdalens Datakonsult AB
+    SPDX-FileCopyrightText: 2004 Klarälvdalens Datakonsult AB
 
-    Libkleopatra is free software; you can redistribute it and/or
-    modify it under the terms of the GNU General Public License as
-    published by the Free Software Foundation; either version 2 of the
-    License, or (at your option) any later version.
-
-    Libkleopatra is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
-
-    In addition, as a special exception, the copyright holders give
-    permission to link the code of this program with any edition of
-    the Qt library by Trolltech AS, Norway (or with modified versions
-    of Qt that use the same license as Qt), and distribute linked
-    combinations including the two.  You must obey the GNU General
-    Public License in all respects for all of the code used other than
-    Qt.  If you modify this file, you may extend this exception to
-    your version of the file, but you are not obligated to do so.  If
-    you do not wish to do so, delete this exception statement from
-    your version.
+    SPDX-License-Identifier: GPL-2.0-or-later
 */
 
 #include "enum.h"
@@ -42,7 +19,6 @@
 #include <gpgme++/tofuinfo.h>
 
 #include <QString>
-#include <QStringList>
 #include <QEventLoop>
 
 static const struct {
@@ -89,20 +65,22 @@ const char *Kleo::cryptoMessageFormatToString(Kleo::CryptoMessageFormat f)
     if (f == AutoFormat) {
         return "auto";
     }
-    for (unsigned int i = 0; i < numCryptoMessageFormats; ++i)
+    for (unsigned int i = 0; i < numCryptoMessageFormats; ++i) {
         if (f == cryptoMessageFormats[i].format) {
             return cryptoMessageFormats[i].configName;
         }
+    }
     return nullptr;
 }
 
 QStringList Kleo::cryptoMessageFormatsToStringList(unsigned int f)
 {
     QStringList result;
-    for (unsigned int i = 0; i < numCryptoMessageFormats; ++i)
+    for (unsigned int i = 0; i < numCryptoMessageFormats; ++i) {
         if (f & cryptoMessageFormats[i].format) {
             result.push_back(QLatin1String(cryptoMessageFormats[i].configName));
         }
+    }
     return result;
 }
 
@@ -111,20 +89,22 @@ QString Kleo::cryptoMessageFormatToLabel(Kleo::CryptoMessageFormat f)
     if (f == AutoFormat) {
         return i18n("Any");
     }
-    for (unsigned int i = 0; i < numCryptoMessageFormats; ++i)
+    for (unsigned int i = 0; i < numCryptoMessageFormats; ++i) {
         if (f == cryptoMessageFormats[i].format) {
             return i18n(cryptoMessageFormats[i].displayName);
         }
+    }
     return QString();
 }
 
 Kleo::CryptoMessageFormat Kleo::stringToCryptoMessageFormat(const QString &s)
 {
     const QString t = s.toLower();
-    for (unsigned int i = 0; i < numCryptoMessageFormats; ++i)
+    for (unsigned int i = 0; i < numCryptoMessageFormats; ++i) {
         if (t == QLatin1String(cryptoMessageFormats[i].configName)) {
             return cryptoMessageFormats[i].format;
         }
+    }
     return AutoFormat;
 }
 
@@ -271,7 +251,7 @@ namespace {
 
 bool hasTrustedSignature(const GpgME::UserID &uid)
 {
-    // lazily intialized cache
+    // lazily initialized cache
     static std::shared_ptr<const Kleo::KeyCache> keyCache;
     if (!keyCache) {
         keyCache = Kleo::KeyCache::instance();
