@@ -2,45 +2,29 @@
     models/keycache.cpp
 
     This file is part of Kleopatra, the KDE keymanager
-    Copyright (c) 2007,2008 Klarälvdalens Datakonsult AB
-                  2018 Intevation GmbH
+    SPDX-FileCopyrightText: 2007, 2008 Klarälvdalens Datakonsult AB
+    SPDX-FileCopyrightText: 2018 Intevation GmbH
+    SPDX-FileCopyrightText: 2020 g10 Code GmbH
+    SPDX-FileContributor: Ingo Klöcker <dev@ingo-kloecker.de>
 
-    Kleopatra is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    Kleopatra is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
-
-    In addition, as a special exception, the copyright holders give
-    permission to link the code of this program with any edition of
-    the Qt library by Trolltech AS, Norway (or with modified versions
-    of Qt that use the same license as Qt), and distribute linked
-    combinations including the two.  You must obey the GNU General
-    Public License in all respects for all of the code used other than
-    Qt.  If you modify this file, you may extend this exception to
-    your version of the file, but you are not obligated to do so.  If
-    you do not wish to do so, delete this exception statement from
-    your version.
+    SPDX-License-Identifier: GPL-2.0-or-later
 */
 
 #include "keycache.h"
 #include "keycache_p.h"
 
-#include "libkleo_debug.h"
-
+#include "kleo/enum.h"
+#include "kleo/keygroup.h"
 #include "kleo/predicates.h"
 #include "kleo/stl_util.h"
 #include "kleo/dn.h"
+
+#include "utils/compat.h"
 #include "utils/filesystemwatcher.h"
 
+#include <KConfigGroup>
+#include <KSharedConfig>
+
 #include <gpgme++/error.h>
 #include <gpgme++/context.h>
 #include <gpgme++/key.h>
@@ -54,8 +38,7 @@
 
 #include <gpg-error.h>
 
-//#include <kmime/kmime_header_parsing.h>
-
+//#include <KMime/HeaderParsing>
 
 #include <QPointer>
 #include <QTimer>
@@ -66,12 +49,18 @@
 #include <functional>
 #include <iterator>
 
+#include "kleo/debug.h"
+#include "libkleo_debug.h"
+#include <chrono>
+using namespace std::chrono_literals;
 using namespace Kleo;
 using namespace GpgME;
 using namespace KMime::Types;
 
 static const unsigned int hours2ms = 1000 * 60 * 60;
 
+static const QString groupNamePrefix = QStringLiteral("Group-");
+
 //
 //
 // KeyCache
@@ -83,6 +72,18 @@ namespace
 
 make_comparator_str(ByEMail, .first.c_str());
 
+QStringList getFingerprints(const KeyGroup::Keys &keys)
+{
+    QStringList fingerprints;
+    fingerprints.reserve(keys.size());
+    std::transform(keys.cbegin(), keys.cend(),
+                    std::back_inserter(fingerprints),
+                    [] (const Key &key) {
+                        return QString::fromLatin1(key.primaryFingerprint());
+                    });
+    return fingerprints;
+}
+
 }
 
 class KeyCache::Private
@@ -107,7 +108,7 @@ public:
     std::vector<Key>::const_iterator find(const std::vector<Key> &keys, const char *key) const
     {
         ensureCachePopulated();
-        const std::vector<Key>::const_iterator it =
+        const auto it =
             std::lower_bound(keys.begin(), keys.end(), key, Comp<std::less>());
         if (it == keys.end() || Comp<std::equal_to>()(*it, key)) {
             return it;
@@ -120,7 +121,7 @@ public:
     std::vector<Subkey>::const_iterator find(const std::vector<Subkey> &keys, const char *key) const
     {
         ensureCachePopulated();
-        const std::vector<Subkey>::const_iterator it =
+        const auto it =
             std::lower_bound(keys.begin(), keys.end(), key, Comp<std::less>());
         if (it == keys.end() || Comp<std::equal_to>()(*it, key)) {
             return it;
@@ -145,6 +146,11 @@ public:
 
     std::vector<Key> find_mailbox(const QString &email, bool sign) const;
 
+    std::vector<Subkey>::const_iterator find_keygrip(const char *keygrip) const
+    {
+        return find<_detail::ByKeyGrip>(by.keygrip, keygrip);
+    }
+
     std::vector<Subkey>::const_iterator find_subkeyid(const char *subkeyid) const
     {
         return find<_detail::ByKeyID>(by.subkeyid, subkeyid);
@@ -198,11 +204,23 @@ public:
 
     void ensureCachePopulated() const;
 
-    void updateGroupCache() {
-        // Update Group Keys
-        // this is a quick thing as it only involves reading the config
-        // so no need for a job.
+    std::vector<Key> getKeysForGroup(const QStringList &fingerprints) const
+    {
+        std::vector<Key> groupKeys;
+        groupKeys.reserve(fingerprints.size());
+        for (const QString &fpr : fingerprints) {
+            const Key key = q->findByFingerprint(fpr.toLatin1().constData());
+            if (key.isNull()) {
+                qCDebug (LIBKLEO_LOG) << "Ignoring unknown key with fingerprint:" << fpr;
+                continue;
+            }
+            groupKeys.push_back(key);
+        }
+        return groupKeys;
+    }
 
+    void readGroupsFromGpgConf()
+    {
         // According to Werner Koch groups are more of a hack to solve
         // a valid usecase (e.g. several keys defined for an internal mailing list)
         // that won't make it in the proper keylist interface. And using gpgconf
@@ -212,30 +230,232 @@ public:
             return;
         }
 
-        auto entry = conf->entry(QStringLiteral("gpg"),
-                                 QStringLiteral("Configuration"),
-                                 QStringLiteral("group"));
+        auto entry = getCryptoConfigEntry(conf, "gpg", "group");
         if (!entry) {
             return;
         }
 
-        m_groups.clear();
-        for (const auto &value: entry->stringValueList()) {
-            const auto split = value.split(QLatin1Char('='));
+        // collect the key fingerprints for all groups read from the configuration
+        QMap<QString, QStringList> fingerprints;
+        const auto stringValueList = entry->stringValueList();
+        for (const QString &value: stringValueList) {
+            const QStringList split = value.split(QLatin1Char('='));
             if (split.size() != 2) {
                 qCDebug (LIBKLEO_LOG) << "Ignoring invalid group config:" << value;
                 continue;
             }
-            const auto name = split[0];
-            const auto key = q->findByFingerprint(split[1].toLatin1().constData());
-            if (key.isNull()) {
-                qCDebug (LIBKLEO_LOG) << "Ignoring unknown fingerprint:" << split[1] << "in group" << name;
-                continue;
-            }
-            auto vec = m_groups.value(name);
-            vec.push_back(key);
-            m_groups.insert(name, vec);
+            const QString groupName = split[0];
+            const QString fingerprint = split[1];
+            fingerprints[groupName].push_back(fingerprint);
         }
+
+        // add all groups read from the configuration to the list of groups
+        for (auto it = fingerprints.cbegin(); it != fingerprints.cend(); ++it) {
+            const QString groupName = it.key();
+            const std::vector<Key> groupKeys = getKeysForGroup(it.value());
+            KeyGroup g(groupName, groupName, groupKeys, KeyGroup::GnuPGConfig);
+            m_groups.push_back(g);
+        }
+    }
+
+    KeyGroup readGroupFromGroupsConfig(const QString &groupId) const
+    {
+        Q_ASSERT(!m_groupsConfigName.isEmpty());
+
+        KSharedConfigPtr groupsConfig = KSharedConfig::openConfig(m_groupsConfigName);
+        const KConfigGroup configGroup = groupsConfig->group(groupNamePrefix + groupId);
+
+        const QString groupName = configGroup.readEntry("Name", QString());
+        const QStringList fingerprints = configGroup.readEntry("Keys", QStringList());
+        const std::vector<Key> groupKeys = getKeysForGroup(fingerprints);
+
+        // treat group as immutable if any of its entries is immutable
+        const QStringList entries = configGroup.keyList();
+        const bool isImmutable = configGroup.isImmutable() || std::any_of(entries.begin(), entries.end(),
+                                                                          [configGroup] (const QString &entry) {
+                                                                              return configGroup.isEntryImmutable(entry);
+                                                                          });
+
+        KeyGroup g(groupId, groupName, groupKeys, KeyGroup::ApplicationConfig);
+        g.setIsImmutable(isImmutable);
+        qCDebug(LIBKLEO_LOG) << "Read group" << g;
+
+        return g;
+    }
+
+    void readGroupsFromGroupsConfig()
+    {
+        if (m_groupsConfigName.isEmpty()) {
+            qCDebug(LIBKLEO_LOG) << "readGroupsFromGroupsConfig - name of application configuration file is unset";
+            return;
+        }
+
+        const KSharedConfigPtr groupsConfig = KSharedConfig::openConfig(m_groupsConfigName);
+        const QStringList configGroups = groupsConfig->groupList();
+        for (const QString &configGroupName : configGroups) {
+            qCDebug(LIBKLEO_LOG) << "Reading config group" << configGroupName;
+            if (configGroupName.startsWith(groupNamePrefix)) {
+                const QString keyGroupId = configGroupName.mid(groupNamePrefix.size());
+                if (keyGroupId.isEmpty()) {
+                    qCWarning(LIBKLEO_LOG) << "Config group" << configGroupName << "has empty group id";
+                    continue;
+                }
+                KeyGroup group = readGroupFromGroupsConfig(keyGroupId);
+                m_groups.push_back(group);
+            }
+        }
+    }
+
+    KeyGroup writeGroupToGroupsConfig(const KeyGroup &group)
+    {
+        if (m_groupsConfigName.isEmpty()) {
+            qCDebug(LIBKLEO_LOG) << "writeGroupToGroupsConfig - name of application configuration file is unset";
+            return KeyGroup();
+        }
+
+        Q_ASSERT(!group.isNull());
+        Q_ASSERT(group.source() == KeyGroup::ApplicationConfig);
+        if (group.isNull() || group.source() != KeyGroup::ApplicationConfig) {
+            qCDebug(LIBKLEO_LOG) << "writeGroupToGroupsConfig - group cannot be written to application configuration:" << group;
+            return KeyGroup();
+        }
+
+        KSharedConfigPtr groupsConfig = KSharedConfig::openConfig(m_groupsConfigName);
+        KConfigGroup configGroup = groupsConfig->group(groupNamePrefix + group.id());
+
+        qCDebug(LIBKLEO_LOG) << "Writing config group" << configGroup.name();
+        configGroup.writeEntry("Name", group.name());
+        configGroup.writeEntry("Keys", getFingerprints(group.keys()));
+
+        // reread group to ensure that it reflects the saved group in case of immutable entries
+        return readGroupFromGroupsConfig(group.id());
+    }
+
+    bool removeGroupFromGroupsConfig(const KeyGroup &group)
+    {
+        if (m_groupsConfigName.isEmpty()) {
+            qCDebug(LIBKLEO_LOG) << "removeGroupFromGroupsConfig - name of application configuration file is unset";
+            return false;
+        }
+
+        Q_ASSERT(!group.isNull());
+        Q_ASSERT(group.source() == KeyGroup::ApplicationConfig);
+        if (group.isNull() || group.source() != KeyGroup::ApplicationConfig) {
+            qCDebug(LIBKLEO_LOG) << "removeGroupFromGroupsConfig - group cannot be removed from application configuration:" << group;
+            return false;
+        }
+
+        KSharedConfigPtr groupsConfig = KSharedConfig::openConfig(m_groupsConfigName);
+        KConfigGroup configGroup = groupsConfig->group(groupNamePrefix + group.id());
+
+        qCDebug(LIBKLEO_LOG) << "Removing config group" << configGroup.name();
+        configGroup.deleteGroup();
+
+        return true;
+    }
+
+    void updateGroupCache()
+    {
+        // Update Group Keys
+        // this is a quick thing as it only involves reading the config
+        // so no need for a job.
+
+        m_groups.clear();
+        if (m_groupsEnabled) {
+            readGroupsFromGpgConf();
+            readGroupsFromGroupsConfig();
+        }
+    }
+
+    bool insert(const KeyGroup &group)
+    {
+        Q_ASSERT(!group.isNull());
+        Q_ASSERT(group.source() == KeyGroup::ApplicationConfig);
+        if (group.isNull() || group.source() != KeyGroup::ApplicationConfig) {
+            qCDebug(LIBKLEO_LOG) << "KeyCache::Private::insert - Invalid group:" << group;
+            return false;
+        }
+        const auto it = std::find_if(m_groups.cbegin(), m_groups.cend(),
+                                    [group] (const auto &g) {
+                                        return g.source() == group.source() && g.id() == group.id();
+                                    });
+        if (it != m_groups.cend()) {
+            qCDebug(LIBKLEO_LOG) << "KeyCache::Private::insert - Group already present in list of groups:" << group;
+            return false;
+        }
+
+        const KeyGroup savedGroup = writeGroupToGroupsConfig(group);
+        if (savedGroup.isNull()) {
+            qCDebug(LIBKLEO_LOG) << "KeyCache::Private::insert - Writing group" << group.id() << "to config file failed";
+            return false;
+        }
+
+        m_groups.push_back(savedGroup);
+
+        Q_EMIT q->groupAdded(savedGroup);
+
+        return true;
+    }
+
+    bool update(const KeyGroup &group)
+    {
+        Q_ASSERT(!group.isNull());
+        Q_ASSERT(group.source() == KeyGroup::ApplicationConfig);
+        if (group.isNull() || group.source() != KeyGroup::ApplicationConfig) {
+            qCDebug(LIBKLEO_LOG) << "KeyCache::Private::update - Invalid group:" << group;
+            return false;
+        }
+        const auto it = std::find_if(m_groups.cbegin(), m_groups.cend(),
+                                    [group] (const auto &g) {
+                                        return g.source() == group.source() && g.id() == group.id();
+                                    });
+        if (it == m_groups.cend()) {
+            qCDebug(LIBKLEO_LOG) << "KeyCache::Private::update - Group not found in list of groups:" << group;
+            return false;
+        }
+        const auto groupIndex = std::distance(m_groups.cbegin(), it);
+
+        const KeyGroup savedGroup = writeGroupToGroupsConfig(group);
+        if (savedGroup.isNull()) {
+            qCDebug(LIBKLEO_LOG) << "KeyCache::Private::update - Writing group" << group.id() << "to config file failed";
+            return false;
+        }
+
+        m_groups[groupIndex] = savedGroup;
+
+        Q_EMIT q->groupUpdated(savedGroup);
+
+        return true;
+    }
+
+    bool remove(const KeyGroup &group)
+    {
+        Q_ASSERT(!group.isNull());
+        Q_ASSERT(group.source() == KeyGroup::ApplicationConfig);
+        if (group.isNull() || group.source() != KeyGroup::ApplicationConfig) {
+            qCDebug(LIBKLEO_LOG) << "KeyCache::Private::remove - Invalid group:" << group;
+            return false;
+        }
+        const auto it = std::find_if(m_groups.cbegin(), m_groups.cend(),
+                                    [group] (const auto &g) {
+                                        return g.source() == group.source() && g.id() == group.id();
+                                    });
+        if (it == m_groups.cend()) {
+            qCDebug(LIBKLEO_LOG) << "KeyCache::Private::remove - Group not found in list of groups:" << group;
+            return false;
+        }
+
+        const bool success = removeGroupFromGroupsConfig(group);
+        if (!success) {
+            qCDebug(LIBKLEO_LOG) << "KeyCache::Private::remove - Removing group" << group.id() << "from config file failed";
+            return false;
+        }
+
+        m_groups.erase(it);
+
+        Q_EMIT q->groupRemoved(group);
+
+        return true;
     }
 
 private:
@@ -247,12 +467,14 @@ private:
     struct By {
         std::vector<Key> fpr, keyid, shortkeyid, chainid;
         std::vector< std::pair<std::string, Key> > email;
-        std::vector<Subkey> subkeyid;
+        std::vector<Subkey> subkeyid, keygrip;
     } by;
     bool m_initalized;
     bool m_pgpOnly;
     bool m_remarks_enabled;
-    QMap <QString, std::vector<Key> > m_groups;
+    bool m_groupsEnabled = false;
+    QString m_groupsConfigName;
+    std::vector<KeyGroup> m_groups;
 };
 
 std::shared_ptr<const KeyCache> KeyCache::instance()
@@ -280,9 +502,22 @@ KeyCache::KeyCache()
 
 KeyCache::~KeyCache() {}
 
+void KeyCache::setGroupsEnabled(bool enabled)
+{
+    d->m_groupsEnabled = enabled;
+    if (d->m_initalized) {
+        d->updateGroupCache();
+    }
+}
+
+void KeyCache::setGroupsConfig(const QString &filename)
+{
+    d->m_groupsConfigName = filename;
+}
+
 void KeyCache::enableFileSystemWatcher(bool enable)
 {
-    for (const auto &i : qAsConst(d->m_fsWatchers)) {
+    for (const auto &i : std::as_const(d->m_fsWatchers)) {
         i->setEnabled(enable);
     }
 }
@@ -347,7 +582,7 @@ void KeyCache::enableRemarks(bool value)
             connect(d->m_refreshJob.data(), &RefreshKeysJob::done,
                     this, [this](const GpgME::KeyListResult &) {
                 qCDebug(LIBKLEO_LOG) << "Reloading keycache with remarks enabled";
-                QTimer::singleShot(1000, this, [this](){ reload(); });
+                QTimer::singleShot(1s, this, [this](){ reload(); });
             });
         }
     } else {
@@ -479,6 +714,31 @@ std::vector<Key> KeyCache::findByKeyIDOrFingerprint(const std::vector<std::strin
     return result;
 }
 
+const Subkey &KeyCache::findSubkeyByKeyGrip(const char *grip, Protocol protocol) const
+{
+    static const Subkey null;
+    d->ensureCachePopulated();
+    const auto range = std::equal_range(d->by.keygrip.begin(), d->by.keygrip.end(),
+                                        grip, _detail::ByKeyGrip<std::less>());
+    if (range.first == d->by.keygrip.end()) {
+        return null;
+    } else if (protocol == UnknownProtocol) {
+        return *range.first;
+    } else {
+        for (auto it = range.first; it != range.second; ++it) {
+            if (it->parent().protocol() == protocol) {
+                return *it;
+            }
+        }
+    }
+    return null;
+}
+
+const Subkey &KeyCache::findSubkeyByKeyGrip(const std::string &grip, Protocol protocol) const
+{
+    return findSubkeyByKeyGrip(grip.c_str(), protocol);
+}
+
 std::vector<Subkey> KeyCache::findSubkeysByKeyID(const std::vector<std::string> &ids) const
 {
     std::vector<std::string> sorted;
@@ -502,10 +762,12 @@ std::vector<Subkey> KeyCache::findSubkeysByKeyID(const std::vector<std::string>
 std::vector<Key> KeyCache::findRecipients(const DecryptionResult &res) const
 {
     std::vector<std::string> keyids;
-    Q_FOREACH (const DecryptionResult::Recipient &r, res.recipients())
+    const auto recipients = res.recipients();
+    for (const DecryptionResult::Recipient &r : recipients) {
         if (const char *kid = r.keyID()) {
             keyids.push_back(kid);
         }
+    }
     const std::vector<Subkey> subkeys = findSubkeysByKeyID(keyids);
     std::vector<Key> result;
     result.reserve(subkeys.size());
@@ -520,10 +782,12 @@ std::vector<Key> KeyCache::findRecipients(const DecryptionResult &res) const
 std::vector<Key> KeyCache::findSigners(const VerificationResult &res) const
 {
     std::vector<std::string> fprs;
-    Q_FOREACH (const Signature &s, res.signatures())
+    const auto signatures = res.signatures();
+    for (const Signature &s : signatures) {
         if (const char *fpr = s.fingerprint()) {
             fprs.push_back(fpr);
         }
+    }
     return findByKeyIDOrFingerprint(fprs);
 }
 
@@ -592,14 +856,15 @@ std::vector<Key> KeyCache::Private::find_mailbox(const QString &email, bool sign
     const auto pair = find_email(email.toUtf8().constData());
     std::vector<Key> result;
     result.reserve(std::distance(pair.first, pair.second));
-    if (sign)
+    if (sign) {
         kdtools::copy_2nd_if(pair.first, pair.second,
                              std::back_inserter(result),
                              ready_for_signing());
-    else
+    } else {
         kdtools::copy_2nd_if(pair.first, pair.second,
                              std::back_inserter(result),
                              ready_for_encryption());
+    }
 
     return result;
 }
@@ -758,7 +1023,8 @@ static std::string email(const UserID &uid)
 static std::vector<std::string> emails(const Key &key)
 {
     std::vector<std::string> emails;
-    Q_FOREACH (const UserID &uid, key.userIDs()) {
+    const auto userIDs = key.userIDs();
+    for (const UserID &uid : userIDs) {
         const std::string e = email(uid);
         if (!e.empty()) {
             emails.push_back(e);
@@ -815,7 +1081,8 @@ void KeyCache::remove(const Key &key)
         d->by.chainid.erase(range2.first, range2.second);
     }
 
-    Q_FOREACH (const std::string &email, emails(key)) {
+    const auto emailsKey{emails(key)};
+    for (const std::string &email : emailsKey) {
         const auto range = std::equal_range(d->by.email.begin(), d->by.email.end(), email, ByEMail<std::less>());
         const auto it = std::remove_if(range.first, range.second,
                                        [fpr](const std::pair<std::string, Key> &pair) {
@@ -824,12 +1091,25 @@ void KeyCache::remove(const Key &key)
         d->by.email.erase(it, range.second);
     }
 
-    Q_FOREACH (const Subkey &subkey, key.subkeys()) {
+    const auto keySubKeys{key.subkeys()};
+    for (const Subkey &subkey : keySubKeys) {
         if (const char *keyid = subkey.keyID()) {
             const auto range = std::equal_range(d->by.subkeyid.begin(), d->by.subkeyid.end(), keyid,
                                                 _detail::ByKeyID<std::less>());
-            const auto range2 = std::equal_range(range.first, range.second, fpr, _detail::ByKeyID<std::less>());
-            d->by.subkeyid.erase(range2.first, range2.second);
+            const auto it = std::remove_if(range.first, range.second,
+                                           [fpr] (const Subkey &subkey) {
+                                               return !qstricmp(fpr, subkey.parent().primaryFingerprint());
+                                           });
+            d->by.subkeyid.erase(it, range.second);
+        }
+        if (const char *keygrip = subkey.keyGrip()) {
+            const auto range = std::equal_range(d->by.keygrip.begin(), d->by.keygrip.end(), keygrip,
+                                                _detail::ByKeyGrip<std::less>());
+            const auto it = std::remove_if(range.first, range.second,
+                                           [fpr] (const Subkey &subkey) {
+                                               return !qstricmp(fpr, subkey.parent().primaryFingerprint());
+                                           });
+            d->by.keygrip.erase(it, range.second);
         }
     }
 }
@@ -856,6 +1136,111 @@ std::vector<Key> KeyCache::secretKeys() const
     return keys;
 }
 
+std::vector<KeyGroup> KeyCache::groups() const
+{
+    d->ensureCachePopulated();
+    return d->m_groups;
+}
+
+std::vector<KeyGroup> KeyCache::configurableGroups() const
+{
+    std::vector<KeyGroup> groups;
+    groups.reserve(d->m_groups.size());
+    std::copy_if(d->m_groups.cbegin(), d->m_groups.cend(),
+                 std::back_inserter(groups),
+                 [] (const KeyGroup &group) { return group.source() == KeyGroup::ApplicationConfig; });
+    return groups;
+}
+
+namespace
+{
+bool compareById(const KeyGroup &lhs, const KeyGroup &rhs)
+{
+    return lhs.id() < rhs.id();
+}
+
+std::vector<KeyGroup> sortedById(std::vector<KeyGroup> groups)
+{
+    std::sort(groups.begin(), groups.end(), &compareById);
+    return groups;
+}
+}
+
+void KeyCache::saveConfigurableGroups(const std::vector<KeyGroup> &groups)
+{
+    const std::vector<KeyGroup> oldGroups = sortedById(configurableGroups());
+    const std::vector<KeyGroup> newGroups = sortedById(groups);
+
+    {
+        std::vector<KeyGroup> removedGroups;
+        std::set_difference(oldGroups.begin(), oldGroups.end(),
+                            newGroups.begin(), newGroups.end(),
+                            std::back_inserter(removedGroups),
+                            &compareById);
+        for (const auto &group : std::as_const(removedGroups)) {
+            qCDebug(LIBKLEO_LOG) << "Removing group" << group;
+            d->remove(group);
+        }
+    }
+    {
+        std::vector<KeyGroup> updatedGroups;
+        std::set_intersection(newGroups.begin(), newGroups.end(),
+                              oldGroups.begin(), oldGroups.end(),
+                              std::back_inserter(updatedGroups),
+                              &compareById);
+        for (const auto &group : std::as_const(updatedGroups)) {
+            qCDebug(LIBKLEO_LOG) << "Updating group" << group;
+            d->update(group);
+        }
+    }
+    {
+        std::vector<KeyGroup> addedGroups;
+        std::set_difference(newGroups.begin(), newGroups.end(),
+                            oldGroups.begin(), oldGroups.end(),
+                            std::back_inserter(addedGroups),
+                            &compareById);
+        for (const auto &group : std::as_const(addedGroups)) {
+            qCDebug(LIBKLEO_LOG) << "Adding group" << group;
+            d->insert(group);
+        }
+    }
+
+    Q_EMIT keysMayHaveChanged();
+}
+
+bool KeyCache::insert(const KeyGroup &group)
+{
+    if (!d->insert(group)) {
+        return false;
+    }
+
+    Q_EMIT keysMayHaveChanged();
+
+    return true;
+}
+
+bool KeyCache::update(const KeyGroup &group)
+{
+    if (!d->update(group)) {
+        return false;
+    }
+
+    Q_EMIT keysMayHaveChanged();
+
+    return true;
+}
+
+bool KeyCache::remove(const KeyGroup &group)
+{
+    if (!d->remove(group)) {
+        return false;
+    }
+
+    Q_EMIT keysMayHaveChanged();
+
+    return true;
+}
+
 void KeyCache::refresh(const std::vector<Key> &keys)
 {
     // make this better...
@@ -876,7 +1261,7 @@ template <
     template <template <typename T> class Op> class T2
     > struct lexicographically
 {
-    typedef bool result_type;
+    using result_type = bool;
 
     template <typename U, typename V>
     bool operator()(const U &lhs, const V &rhs) const
@@ -922,7 +1307,7 @@ void KeyCache::insert(const std::vector<Key> &keys)
     // 3. build email index:
     std::vector< std::pair<std::string, Key> > pairs;
     pairs.reserve(sorted.size());
-    Q_FOREACH (const Key &key, sorted) {
+    for (const Key &key : std::as_const(sorted)) {
         const std::vector<std::string> emails = ::emails(key);
         for (const std::string &e : emails) {
             pairs.push_back(std::make_pair(e, key));
@@ -979,31 +1364,45 @@ void KeyCache::insert(const std::vector<Key> &keys)
     // 6. build subkey ID index:
     std::vector<Subkey> subkeys;
     subkeys.reserve(sorted.size());
-    Q_FOREACH (const Key &key, sorted)
-        Q_FOREACH (const Subkey &subkey, key.subkeys()) {
+    for (const Key &key : std::as_const(sorted)) {
+        const auto keySubkeys{key.subkeys()};
+        for (const Subkey &subkey : keySubkeys) {
             subkeys.push_back(subkey);
         }
+    }
 
     // 6a sort by key id:
     std::sort(subkeys.begin(), subkeys.end(), _detail::ByKeyID<std::less>());
 
     // 6b. insert into subkey ID index:
     std::vector<Subkey> by_subkeyid;
-    by_email.reserve(subkeys.size() + d->by.subkeyid.size());
+    by_subkeyid.reserve(subkeys.size() + d->by.subkeyid.size());
     std::merge(subkeys.begin(), subkeys.end(),
                d->by.subkeyid.begin(), d->by.subkeyid.end(),
                std::back_inserter(by_subkeyid),
                _detail::ByKeyID<std::less>());
 
+    // 6c. sort by key grip
+    std::sort(subkeys.begin(), subkeys.end(), _detail::ByKeyGrip<std::less>());
+
+    // 6d. insert into subkey keygrip index:
+    std::vector<Subkey> by_keygrip;
+    by_keygrip.reserve(subkeys.size() + d->by.keygrip.size());
+    std::merge(subkeys.begin(), subkeys.end(),
+               d->by.keygrip.begin(), d->by.keygrip.end(),
+               std::back_inserter(by_keygrip),
+               _detail::ByKeyGrip<std::less>());
+
     // now commit (well, we already removed keys...)
     by_fpr.swap(d->by.fpr);
     by_keyid.swap(d->by.keyid);
     by_shortkeyid.swap(d->by.shortkeyid);
     by_email.swap(d->by.email);
     by_subkeyid.swap(d->by.subkeyid);
+    by_keygrip.swap(d->by.keygrip);
     by_chainid.swap(d->by.chainid);
 
-    for (const Key &key : qAsConst(sorted)) {
+    for (const Key &key : std::as_const(sorted)) {
         d->m_pgpOnly &= key.protocol() == GpgME::OpenPGP;
         Q_EMIT added(key);
     }
@@ -1035,11 +1434,12 @@ public:
         keys.reserve(m_keys.size() + nextKeys.size());
         if (m_keys.empty()) {
             keys = nextKeys;
-        } else
+        } else {
             std::merge(m_keys.begin(), m_keys.end(),
                        nextKeys.begin(), nextKeys.end(),
                        std::back_inserter(keys),
                        _detail::ByFingerprint<std::less>());
+        }
         m_keys.swap(keys);
         jobDone(res);
     }
@@ -1242,90 +1642,170 @@ static bool subkeyIsOk(const Subkey &s)
     return !s.isRevoked() && !s.isInvalid() && !s.isDisabled();
 }
 
-std::vector<GpgME::Key> KeyCache::findBestByMailBox(const char *addr, GpgME::Protocol proto,
-                                                    bool needSign, bool needEncrypt) const
+namespace
 {
-    d->ensureCachePopulated();
-    std::vector<GpgME::Key> ret;
-    if (!addr) {
-        return ret;
-    }
-    if (proto == Protocol::OpenPGP) {
-        // Groups take precedence over automatic keys. We return all here
-        // so if a group key for example is expired we can show that.
-        ret = getGroupKeys(QString::fromUtf8(addr));
-        if (!ret.empty()) {
-            return ret;
+time_t creationTimeOfNewestSuitableSubKey(const Key &key, KeyUsage usage)
+{
+    time_t creationTime = 0;
+    for (const Subkey &s: key.subkeys()) {
+        if (!subkeyIsOk(s)) {
+            continue;
+        }
+        if (usage == KeyUsage::Sign && !s.canSign()) {
+            continue;
+        }
+        if (usage == KeyUsage::Encrypt && !s.canEncrypt()) {
+            continue;
+        }
+        if (s.creationTime() > creationTime) {
+            creationTime = s.creationTime();
         }
     }
-    Key keyC;
-    UserID uidC;
-    for (const Key &k: findByEMailAddress(addr)) {
+    return creationTime;
+}
+
+struct BestMatch
+{
+    Key key;
+    UserID uid;
+    time_t creationTime = 0;
+};
+}
+
+GpgME::Key KeyCache::findBestByMailBox(const char *addr, GpgME::Protocol proto, KeyUsage usage) const
+{
+    d->ensureCachePopulated();
+    if (!addr) {
+        return {};
+    }
+
+    // support lookup of email addresses enclosed in angle brackets
+    QByteArray address(addr);
+    if (address[0] == '<' && address[address.size() - 1] == '>') {
+        address = address.mid(1, address.size() - 2);
+    }
+    address = address.toLower();
+
+    BestMatch best;
+    for (const Key &k: findByEMailAddress(address.constData())) {
         if (proto != Protocol::UnknownProtocol && k.protocol() != proto) {
             continue;
         }
-        if (needEncrypt && !k.canEncrypt()) {
+        if (usage == KeyUsage::Encrypt && !k.canEncrypt()) {
             continue;
         }
-        if (needSign && (!k.canSign() || !k.hasSecret())) {
+        if (usage == KeyUsage::Sign && (!k.canSign() || !k.hasSecret())) {
+            continue;
+        }
+        const time_t creationTime = creationTimeOfNewestSuitableSubKey(k, usage);
+        if (creationTime == 0) {
+            // key does not have a suitable (and usable) subkey
             continue;
         }
-        /* First get the uid that matches the mailbox */
         for (const UserID &u: k.userIDs()) {
-            if (QString::fromStdString(u.addrSpec()).toLower() != QString::fromUtf8(addr).toLower()) {
+            if (QByteArray::fromStdString(u.addrSpec()).toLower() != address) {
+                // user ID does not match the given email address
                 continue;
             }
-            if (uidC.isNull()) {
-                keyC = k;
-                uidC = u;
-            } else if ((!uidIsOk(uidC) && uidIsOk(u)) || uidC.validity() < u.validity()) {
-                /* Validity of the new key is better. */
-                uidC = u;
-                keyC = k;
-            } else if (uidC.validity() == u.validity() && uidIsOk(u)) {
-                /* Both are the same check which one is newer. */
-                time_t oldTime = 0;
-                for (const Subkey &s: keyC.subkeys()) {
-                    if (!subkeyIsOk(s)) {
-                        continue;
-                    }
-                    if (needSign && s.canSign()) {
-                        oldTime = s.creationTime();
-                    } else if (needEncrypt && s.canEncrypt()) {
-                        oldTime = s.creationTime();
-                    } else if (s.canSign() || s.canEncrypt()) {
-                        oldTime = s.creationTime();
-                    }
-                }
-                time_t newTime = 0;
-                for (const Subkey &s: k.subkeys()) {
-                    if (!subkeyIsOk(s)) {
-                        continue;
-                    }
-                    if (needSign && s.canSign()) {
-                        newTime = s.creationTime();
-                    } else if (needEncrypt && s.canEncrypt()) {
-                        newTime = s.creationTime();
-                    } else if (s.canSign() || s.canEncrypt()) {
-                        newTime = s.creationTime();
-                    }
-                }
-                if (newTime > oldTime) {
-                    uidC = u;
-                    keyC = k;
-                }
+            if (best.uid.isNull()) {
+                // we have found our first candidate
+                best = {k, u, creationTime};
+            } else if (!uidIsOk(best.uid) && uidIsOk(u)) {
+                // validity of the new key is better
+                best = {k, u, creationTime};
+            } else if (!k.isExpired() && best.uid.validity() < u.validity()) {
+                // validity of the new key is better
+                best = {k, u, creationTime};
+            } else if (best.key.isExpired() && !k.isExpired()) {
+                // validity of the new key is better
+                best = {k, u, creationTime};
+            } else if (best.uid.validity() == u.validity() && uidIsOk(u) && best.creationTime < creationTime) {
+                // both keys/user IDs have same validity, but the new key is newer
+                best = {k, u, creationTime};
             }
         }
     }
-    ret.push_back(keyC);
-    return ret;
+
+    return best.key;
 }
 
-std::vector<GpgME::Key> KeyCache::getGroupKeys(const QString &groupName) const
+namespace
 {
-    return d->m_groups.value(groupName);
+template<typename T>
+bool allKeysAllowUsage(const T &keys, KeyUsage usage)
+{
+    switch(usage) {
+    case KeyUsage::AnyUsage:
+        return true;
+    case KeyUsage::Sign:
+        return std::all_of(std::begin(keys), std::end(keys), std::mem_fn(&Key::canSign));
+    case KeyUsage::Encrypt:
+        return std::all_of(std::begin(keys), std::end(keys), std::mem_fn(&Key::canEncrypt));
+    case KeyUsage::Certify:
+        return std::all_of(std::begin(keys), std::end(keys), std::mem_fn(&Key::canCertify));
+    case KeyUsage::Authenticate:
+        return std::all_of(std::begin(keys), std::end(keys), std::mem_fn(&Key::canAuthenticate));
+    }
+    qCDebug(LIBKLEO_LOG) << __func__ << "called with invalid usage" << int(usage);
+    return false;
+}
+
+template<typename T>
+bool allKeysHaveProtocol(const T &keys, Protocol protocol)
+{
+    return std::all_of(std::begin(keys), std::end(keys), [protocol] (const auto &key) { return key.protocol() == protocol; });
+}
+}
+
+KeyGroup KeyCache::findGroup(const QString &name, Protocol protocol, KeyUsage usage) const
+{
+    d->ensureCachePopulated();
+
+    Q_ASSERT(usage == KeyUsage::Sign || usage == KeyUsage::Encrypt);
+    for (const auto &group : std::as_const(d->m_groups)) {
+        if (group.name() == name) {
+            const KeyGroup::Keys &keys = group.keys();
+            if (allKeysAllowUsage(keys, usage)
+                && (protocol == UnknownProtocol || allKeysHaveProtocol(keys, protocol))) {
+                return group;
+            }
+        }
+    }
+
+    return {};
+}
+
+std::vector<Key> KeyCache::getGroupKeys(const QString &groupName) const
+{
+    std::vector<Key> result;
+    for (const KeyGroup &g : std::as_const(d->m_groups)) {
+        if (g.name() == groupName) {
+            const KeyGroup::Keys &keys = g.keys();
+            std::copy(keys.cbegin(), keys.cend(), std::back_inserter(result));
+        }
+    }
+    _detail::sort_by_fpr(result);
+    _detail::remove_duplicates_by_fpr(result);
+    return result;
+}
+
+void KeyCache::setKeys(const std::vector<GpgME::Key>& keys)
+{
+    // disable regular key listing and cancel running key listing
+    setRefreshInterval(0);
+    cancelKeyListing();
+    clear();
+    insert(keys);
+    d->m_initalized = true;
+    Q_EMIT keyListingDone(KeyListResult());
+}
+
+void KeyCache::setGroups(const std::vector<KeyGroup>& groups)
+{
+    Q_ASSERT(d->m_initalized && "Call setKeys() before setting groups");
+    d->m_groups = groups;
+    Q_EMIT keysMayHaveChanged();
 }
 
 #include "moc_keycache_p.cpp"
 #include "moc_keycache.cpp"
-
