@@ -1,40 +1,22 @@
 /* -*- mode: c++; c-basic-offset:4 -*-
     models/keylistmodel.cpp
 
-    This file is part of Kleopatra, the KDE keymanager
-    Copyright (c) 2007 Klarälvdalens Datakonsult AB
+    This file is part of libkleopatra, the KDE keymanagement library
+    SPDX-FileCopyrightText: 2007 Klarälvdalens Datakonsult AB
+    SPDX-FileCopyrightText: 2021 g10 Code GmbH
+    SPDX-FileContributor: Ingo Klöcker <dev@ingo-kloecker.de>
 
-    Kleopatra is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    Kleopatra is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
-
-    In addition, as a special exception, the copyright holders give
-    permission to link the code of this program with any edition of
-    the Qt library by Trolltech AS, Norway (or with modified versions
-    of Qt that use the same license as Qt), and distribute linked
-    combinations including the two.  You must obey the GNU General
-    Public License in all respects for all of the code used other than
-    Qt.  If you modify this file, you may extend this exception to
-    your version of the file, but you are not obligated to do so.  If
-    you do not wish to do so, delete this exception statement from
-    your version.
+    SPDX-License-Identifier: GPL-2.0-or-later
 */
 
 #include "keylistmodel.h"
+
 #include "keycache.h"
+#include "kleo/keygroup.h"
 #include "kleo/predicates.h"
 #include "kleo/keyfiltermanager.h"
 #include "kleo/keyfilter.h"
+#include "utils/algorithm.h"
 #include "utils/formatting.h"
 
 #ifdef KLEO_MODEL_TEST
@@ -59,7 +41,6 @@
 #endif
 
 #include <algorithm>
-#include <vector>
 #include <map>
 #include <set>
 #include <iterator>
@@ -71,27 +52,49 @@
 
 using namespace GpgME;
 using namespace Kleo;
+using namespace Kleo::KeyList;
 
 Q_DECLARE_METATYPE(GpgME::Key)
+Q_DECLARE_METATYPE(KeyGroup)
 
 class AbstractKeyListModel::Private
 {
+    AbstractKeyListModel *const q;
 public:
-    Private() :
-        m_toolTipOptions(Formatting::Validity),
-        m_useKeyCache(false),
-        m_secretOnly(false) {}
-    int m_toolTipOptions;
+    explicit Private(AbstractKeyListModel *qq);
+
+    void updateFromKeyCache();
+
+public:
+    int m_toolTipOptions = Formatting::Validity;
     mutable QHash<const char *, QVariant> prettyEMailCache;
     mutable QHash<const char *, QVariant> remarksCache;
-    bool m_useKeyCache;
-    bool m_secretOnly;
+    bool m_useKeyCache = false;
+    bool m_modelResetInProgress = false;
+    KeyList::Options m_keyListOptions = AllKeys;
     std::vector<GpgME::Key> m_remarkKeys;
 };
-AbstractKeyListModel::AbstractKeyListModel(QObject *p)
-    : QAbstractItemModel(p), KeyListModelInterface(), d(new Private)
-{
 
+AbstractKeyListModel::Private::Private(Kleo::AbstractKeyListModel *qq)
+    : q(qq)
+{
+}
+
+void AbstractKeyListModel::Private::updateFromKeyCache()
+{
+    if (m_useKeyCache) {
+        q->setKeys(m_keyListOptions == SecretKeysOnly ? KeyCache::instance()->secretKeys() : KeyCache::instance()->keys());
+        if (m_keyListOptions == IncludeGroups) {
+            q->setGroups(KeyCache::instance()->groups());
+        }
+    }
+}
+
+AbstractKeyListModel::AbstractKeyListModel(QObject *p)
+    : QAbstractItemModel(p), KeyListModelInterface(), d(new Private(this))
+{
+    connect(this, &QAbstractItemModel::modelAboutToBeReset, this, [this] () { d->m_modelResetInProgress = true; });
+    connect(this, &QAbstractItemModel::modelReset, this, [this] () { d->m_modelResetInProgress = false; });
 }
 
 AbstractKeyListModel::~AbstractKeyListModel() {}
@@ -134,14 +137,29 @@ std::vector<Key> AbstractKeyListModel::keys(const QList<QModelIndex> &indexes) c
                    [this](const QModelIndex &idx) {
                        return this->key(idx);
                    });
-    result.erase(std::unique(result.begin(), result.end(), _detail::ByFingerprint<std::equal_to>()), result.end());
+    result.erase(std::remove_if(result.begin(), result.end(), std::mem_fn(&GpgME::Key::isNull)), result.end());
+    _detail::remove_duplicates_by_fpr(result);
     return result;
 }
 
+KeyGroup AbstractKeyListModel::group(const QModelIndex &idx) const
+{
+    if (idx.isValid()) {
+        return doMapToGroup(idx);
+    } else {
+        return KeyGroup();
+    }
+}
+
+QModelIndex AbstractKeyListModel::index(const Key &key) const
+{
+    return index(key, 0);
+}
+
 QModelIndex AbstractKeyListModel::index(const Key &key, int col) const
 {
     if (key.isNull() || col < 0 || col >= NumColumns) {
-        return QModelIndex();
+        return {};
     } else {
         return doMapFromKey(key, col);
     }
@@ -159,10 +177,26 @@ QList<QModelIndex> AbstractKeyListModel::indexes(const std::vector<Key> &keys) c
     return result;
 }
 
+QModelIndex AbstractKeyListModel::index(const KeyGroup &group) const
+{
+    return index(group, 0);
+}
+
+QModelIndex AbstractKeyListModel::index(const KeyGroup &group, int col) const
+{
+    if (group.isNull() || col < 0 || col >= NumColumns) {
+        return {};
+    } else {
+        return doMapFromGroup(group, col);
+    }
+}
+
 void AbstractKeyListModel::setKeys(const std::vector<Key> &keys)
 {
-    clear();
+    beginResetModel();
+    clear(Keys);
     addKeys(keys);
+    endResetModel();
 }
 
 QModelIndex AbstractKeyListModel::addKey(const Key &key)
@@ -193,15 +227,46 @@ QList<QModelIndex> AbstractKeyListModel::addKeys(const std::vector<Key> &keys)
     return doAddKeys(sorted);
 }
 
-void AbstractKeyListModel::clear()
+void AbstractKeyListModel::setGroups(const std::vector<KeyGroup> &groups)
 {
     beginResetModel();
-    doClear();
-    d->prettyEMailCache.clear();
-    d->remarksCache.clear();
+    clear(Groups);
+    doSetGroups(groups);
     endResetModel();
 }
 
+QModelIndex AbstractKeyListModel::addGroup(const KeyGroup &group)
+{
+    if (group.isNull()) {
+        return QModelIndex();
+    }
+    return doAddGroup(group);
+}
+
+bool AbstractKeyListModel::removeGroup(const KeyGroup &group)
+{
+    if (group.isNull()) {
+        return false;
+    }
+    return doRemoveGroup(group);
+}
+
+void AbstractKeyListModel::clear(ItemTypes types)
+{
+    const bool inReset = modelResetInProgress();
+    if (!inReset) {
+        beginResetModel();
+    }
+    doClear(types);
+    if (types & Keys) {
+        d->prettyEMailCache.clear();
+        d->remarksCache.clear();
+    }
+    if (!inReset) {
+        endResetModel();
+    }
+}
+
 int AbstractKeyListModel::columnCount(const QModelIndex &) const
 {
     return NumColumns;
@@ -209,8 +274,8 @@ int AbstractKeyListModel::columnCount(const QModelIndex &) const
 
 QVariant AbstractKeyListModel::headerData(int section, Qt::Orientation o, int role) const
 {
-    if (o == Qt::Horizontal)
-        if (role == Qt::DisplayRole || role == Qt::EditRole || role == Qt::ToolTipRole)
+    if (o == Qt::Horizontal) {
+        if (role == Qt::DisplayRole || role == Qt::EditRole || role == Qt::ToolTipRole) {
             switch (section) {
             case PrettyName:       return i18n("Name");
             case PrettyEMail:      return i18n("E-Mail");
@@ -229,6 +294,8 @@ QVariant AbstractKeyListModel::headerData(int section, Qt::Orientation o, int ro
             case Remarks:          return i18n("Tags");
             case NumColumns:;
             }
+        }
+    }
     return QVariant();
 }
 
@@ -253,13 +320,21 @@ static QVariant returnIfValid(const QIcon &t)
 QVariant AbstractKeyListModel::data(const QModelIndex &index, int role) const
 {
     const Key key = this->key(index);
-    if (key.isNull()) {
-        return QVariant();
+    if (!key.isNull()) {
+        return data(key, index.column(), role);
     }
 
-    const int column = index.column();
+    const KeyGroup group = this->group(index);
+    if (!group.isNull()) {
+        return data(group, index.column(), role);
+    }
 
-    if (role == Qt::DisplayRole || role == Qt::EditRole) {
+    return QVariant();
+}
+
+QVariant AbstractKeyListModel::data(const Key &key, int column, int role) const
+{
+   if (role == Qt::DisplayRole || role == Qt::EditRole) {
         switch (column) {
         case PrettyName:
             return Formatting::prettyName(key);
@@ -328,6 +403,7 @@ QVariant AbstractKeyListModel::data(const QModelIndex &index, int role) const
                             return d->remarksCache[fpr] = remark;
                         } else {
                             QStringList remarkList;
+                            remarkList.reserve(remarks.size());
                             for (const auto &rem: remarks) {
                                 remarkList << QString::fromStdString(rem);
                             }
@@ -362,6 +438,76 @@ QVariant AbstractKeyListModel::data(const QModelIndex &index, int role) const
     return QVariant();
 }
 
+QVariant AbstractKeyListModel::data(const KeyGroup &group, int column, int role) const
+{
+    if (role == Qt::DisplayRole || role == Qt::EditRole) {
+        switch (column) {
+        case PrettyName:
+            return group.name();
+        case PrettyEMail:
+            return QVariant();
+        case Validity:
+            return Formatting::complianceStringShort(group);
+        case ValidFrom:
+            return QString();
+        case ValidUntil:
+            return QString();
+        case TechnicalDetails:
+            return Formatting::type(group);
+        case ShortKeyID:
+            return QString();
+        case KeyID:
+            return QString();
+        case Summary:
+            return Formatting::summaryLine(group);  // used for filtering
+        case Fingerprint:
+            return QString();
+        case Issuer:
+            return QString();
+        case Origin:
+            return QString();
+        case LastUpdate:
+            return QString();
+        case SerialNumber:
+            return QString();
+        case OwnerTrust:
+            return QString();
+        case Remarks:
+            return QVariant();
+        case NumColumns:
+            break;
+        }
+    } else if (role == Qt::ToolTipRole) {
+        return Formatting::toolTip(group, toolTipOptions());
+    } else if (role == Qt::FontRole) {
+        return QFont();
+    } else if (role == Qt::DecorationRole) {
+        return column == Icon ? QIcon::fromTheme(QStringLiteral("group")) : QVariant();
+    } else if (role == Qt::BackgroundRole) {
+    } else if (role == Qt::ForegroundRole) {
+    } else if (role == GroupRole) {
+        return QVariant::fromValue(group);
+    }
+    return QVariant();
+}
+
+bool AbstractKeyListModel::setData(const QModelIndex &index, const QVariant &value, int role)
+{
+    Q_UNUSED(role)
+    Q_ASSERT(value.canConvert<KeyGroup>());
+    if (value.canConvert<KeyGroup>()) {
+        const KeyGroup group = value.value<KeyGroup>();
+        return doSetGroupData(index, group);
+    }
+
+    return false;
+}
+
+bool AbstractKeyListModel::modelResetInProgress()
+{
+    return d->m_modelResetInProgress;
+}
+
 namespace
 {
 template <typename Base>
@@ -369,7 +515,7 @@ class TableModelMixin : public Base
 {
 public:
     explicit TableModelMixin(QObject *p = nullptr) : Base(p) {}
-    ~TableModelMixin() {}
+    ~TableModelMixin() override {}
 
     using Base::index;
     QModelIndex index(int row, int column, const QModelIndex &pidx = QModelIndex()) const override
@@ -402,7 +548,7 @@ public:
 
     int rowCount(const QModelIndex &pidx) const override
     {
-        return pidx.isValid() ? 0 : mKeysByFingerprint.size();
+        return pidx.isValid() ? 0 : mKeysByFingerprint.size() + mGroups.size();
     }
 
 private:
@@ -410,12 +556,47 @@ private:
     QModelIndex doMapFromKey(const Key &key, int col) const override;
     QList<QModelIndex> doAddKeys(const std::vector<Key> &keys) override;
     void doRemoveKey(const Key &key) override;
-    void doClear() override {
-        mKeysByFingerprint.clear();
+
+    KeyGroup doMapToGroup(const QModelIndex &index) const override;
+    QModelIndex doMapFromGroup(const KeyGroup &group, int column) const override;
+    void doSetGroups(const std::vector<KeyGroup> &groups) override;
+    QModelIndex doAddGroup(const KeyGroup &group) override;
+    bool doSetGroupData(const QModelIndex &index, const KeyGroup &group) override;
+    bool doRemoveGroup(const KeyGroup &group) override;
+
+    void doClear(ItemTypes types) override
+    {
+        if (types & Keys) {
+            mKeysByFingerprint.clear();
+        }
+        if (types & Groups) {
+            mGroups.clear();
+        }
+    }
+
+    int firstGroupRow() const
+    {
+        return mKeysByFingerprint.size();
+    }
+
+    int lastGroupRow() const
+    {
+        return mKeysByFingerprint.size() + mGroups.size() - 1;
+    }
+
+    int groupIndex(const QModelIndex &index) const
+    {
+        if (!index.isValid()
+                || index.row() < firstGroupRow() || index.row() > lastGroupRow()
+                || index.column() >= NumColumns) {
+            return -1;
+        }
+        return index.row() - firstGroupRow();
     }
 
 private:
     std::vector<Key> mKeysByFingerprint;
+    std::vector<KeyGroup> mGroups;
 };
 
 class HierarchicalKeyListModel : public AbstractKeyListModel
@@ -440,11 +621,45 @@ private:
     QModelIndex doMapFromKey(const Key &key, int col) const override;
     QList<QModelIndex> doAddKeys(const std::vector<Key> &keys) override;
     void doRemoveKey(const Key &key) override;
-    void doClear() override {
-        mTopLevels.clear();
-        mKeysByFingerprint.clear();
-        mKeysByExistingParent.clear();
-        mKeysByNonExistingParent.clear();
+
+    KeyGroup doMapToGroup(const QModelIndex &index) const override;
+    QModelIndex doMapFromGroup(const KeyGroup &group, int column) const override;
+    void doSetGroups(const std::vector<KeyGroup> &groups) override;
+    QModelIndex doAddGroup(const KeyGroup &group) override;
+    bool doSetGroupData(const QModelIndex &index, const KeyGroup &group) override;
+    bool doRemoveGroup(const KeyGroup &group) override;
+
+    void doClear(ItemTypes types) override
+    {
+        if (types & Keys) {
+            mTopLevels.clear();
+            mKeysByFingerprint.clear();
+            mKeysByExistingParent.clear();
+            mKeysByNonExistingParent.clear();
+        }
+        if (types & Groups) {
+            mGroups.clear();
+        }
+    }
+
+    int firstGroupRow() const
+    {
+        return mTopLevels.size();
+    }
+
+    int lastGroupRow() const
+    {
+        return mTopLevels.size() + mGroups.size() - 1;
+    }
+
+    int groupIndex(const QModelIndex &index) const
+    {
+        if (!index.isValid()
+                || index.row() < firstGroupRow() || index.row() > lastGroupRow()
+                || index.column() >= NumColumns) {
+            return -1;
+        }
+        return index.row() - firstGroupRow();
     }
 
 private:
@@ -457,6 +672,7 @@ private:
     std::vector<Key> mKeysByFingerprint; // all keys
     Map mKeysByExistingParent, mKeysByNonExistingParent; // parent->child map
     std::vector<Key> mTopLevels; // all roots + parent-less
+    std::vector<KeyGroup> mGroups;
 };
 
 static const char *cleanChainID(const Key &key)
@@ -473,10 +689,8 @@ static const char *cleanChainID(const Key &key)
 }
 
 FlatKeyListModel::FlatKeyListModel(QObject *p)
-    : TableModelMixin<AbstractKeyListModel>(p),
-      mKeysByFingerprint()
+    : TableModelMixin<AbstractKeyListModel>(p)
 {
-
 }
 
 FlatKeyListModel::~FlatKeyListModel() {}
@@ -498,7 +712,7 @@ QModelIndex FlatKeyListModel::doMapFromKey(const Key &key, int col) const
         = std::lower_bound(mKeysByFingerprint.begin(), mKeysByFingerprint.end(),
                            key, _detail::ByFingerprint<std::less>());
     if (it == mKeysByFingerprint.end() || !_detail::ByFingerprint<std::equal_to>()(*it, key)) {
-        return QModelIndex();
+        return {};
     } else {
         return createIndex(it - mKeysByFingerprint.begin(), col);
     }
@@ -512,7 +726,7 @@ QList<QModelIndex> FlatKeyListModel::doAddKeys(const std::vector<Key> &keys)
         return QList<QModelIndex>();
     }
 
-    for (std::vector<Key>::const_iterator it = keys.begin(), end = keys.end(); it != end; ++it) {
+    for (auto it = keys.begin(), end = keys.end(); it != end; ++it) {
 
         // find an insertion point:
         const std::vector<Key>::iterator pos = std::upper_bound(mKeysByFingerprint.begin(), mKeysByFingerprint.end(), *it, _detail::ByFingerprint<std::less>());
@@ -521,12 +735,18 @@ QList<QModelIndex> FlatKeyListModel::doAddKeys(const std::vector<Key> &keys)
         if (idx > 0 && qstrcmp(mKeysByFingerprint[idx - 1].primaryFingerprint(), it->primaryFingerprint()) == 0) {
             // key existed before - replace with new one:
             mKeysByFingerprint[idx - 1] = *it;
-            Q_EMIT dataChanged(createIndex(idx - 1, 0), createIndex(idx - 1, NumColumns - 1));
+            if (!modelResetInProgress()) {
+                Q_EMIT dataChanged(createIndex(idx - 1, 0), createIndex(idx - 1, NumColumns - 1));
+            }
         } else {
             // new key - insert:
-            beginInsertRows(QModelIndex(), idx, idx);
+            if (!modelResetInProgress()) {
+                beginInsertRows(QModelIndex(), idx, idx);
+            }
             mKeysByFingerprint.insert(pos, *it);
-            endInsertRows();
+            if (!modelResetInProgress()) {
+                endInsertRows();
+            }
         }
     }
 
@@ -536,16 +756,110 @@ QList<QModelIndex> FlatKeyListModel::doAddKeys(const std::vector<Key> &keys)
 void FlatKeyListModel::doRemoveKey(const Key &key)
 {
     const std::vector<Key>::iterator it
-        = qBinaryFind(mKeysByFingerprint.begin(), mKeysByFingerprint.end(),
-                      key, _detail::ByFingerprint<std::less>());
+        = Kleo::binary_find(mKeysByFingerprint.begin(), mKeysByFingerprint.end(),
+                            key, _detail::ByFingerprint<std::less>());
     if (it == mKeysByFingerprint.end()) {
         return;
     }
 
     const unsigned int row = std::distance(mKeysByFingerprint.begin(), it);
-    beginRemoveRows(QModelIndex(), row, row);
+    if (!modelResetInProgress()) {
+        beginRemoveRows(QModelIndex(), row, row);
+    }
     mKeysByFingerprint.erase(it);
-    endRemoveRows();
+    if (!modelResetInProgress()) {
+        endRemoveRows();
+    }
+}
+
+KeyGroup FlatKeyListModel::doMapToGroup(const QModelIndex &idx) const
+{
+    Q_ASSERT(idx.isValid());
+    if (static_cast<unsigned>(idx.row()) >= mKeysByFingerprint.size()
+            && static_cast<unsigned>(idx.row()) < mKeysByFingerprint.size() + mGroups.size()
+            && idx.column() < NumColumns) {
+        return mGroups[ idx.row() - mKeysByFingerprint.size() ];
+    } else {
+        return KeyGroup();
+    }
+}
+
+QModelIndex FlatKeyListModel::doMapFromGroup(const KeyGroup &group, int column) const
+{
+    Q_ASSERT(!group.isNull());
+    const auto it = std::find_if(mGroups.cbegin(), mGroups.cend(),
+                                 [group](const KeyGroup &g) {
+                                     return g.source() == group.source() && g.id() == group.id();
+                                 });
+    if (it == mGroups.cend()) {
+        return QModelIndex();
+    } else {
+        return createIndex(it - mGroups.cbegin() + mKeysByFingerprint.size(), column);
+    }
+}
+
+void FlatKeyListModel::doSetGroups(const std::vector<KeyGroup> &groups)
+{
+    Q_ASSERT(mGroups.empty());  // ensure that groups have been cleared
+    const int first = mKeysByFingerprint.size();
+    const int last = first + groups.size() - 1;
+    if (!modelResetInProgress()) {
+        beginInsertRows(QModelIndex(), first, last);
+    }
+    mGroups = groups;
+    if (!modelResetInProgress()) {
+        endInsertRows();
+    }
+}
+
+QModelIndex FlatKeyListModel::doAddGroup(const KeyGroup &group)
+{
+    const int newRow = lastGroupRow() + 1;
+    if (!modelResetInProgress()) {
+        beginInsertRows(QModelIndex(), newRow, newRow);
+    }
+    mGroups.push_back(group);
+    if (!modelResetInProgress()) {
+        endInsertRows();
+    }
+    return createIndex(newRow, 0);
+}
+
+bool FlatKeyListModel::doSetGroupData(const QModelIndex &index, const KeyGroup &group)
+{
+    if (group.isNull()) {
+        return false;
+    }
+    const int groupIndex = this->groupIndex(index);
+    if (groupIndex == -1) {
+        return false;
+    }
+    mGroups[groupIndex] = group;
+    if (!modelResetInProgress()) {
+        Q_EMIT dataChanged(createIndex(index.row(), 0), createIndex(index.row(), NumColumns - 1));
+    }
+    return true;
+}
+
+bool FlatKeyListModel::doRemoveGroup(const KeyGroup &group)
+{
+    const QModelIndex modelIndex = doMapFromGroup(group, 0);
+    if (!modelIndex.isValid()) {
+        return false;
+    }
+    const int groupIndex = this->groupIndex(modelIndex);
+    Q_ASSERT(groupIndex != -1);
+    if (groupIndex == -1) {
+        return false;
+    }
+    if (!modelResetInProgress()) {
+        beginRemoveRows(QModelIndex(), modelIndex.row(), modelIndex.row());
+    }
+    mGroups.erase(mGroups.begin() + groupIndex);
+    if (!modelResetInProgress()) {
+        endRemoveRows();
+    }
+    return true;
 }
 
 HierarchicalKeyListModel::HierarchicalKeyListModel(QObject *p)
@@ -565,7 +879,7 @@ int HierarchicalKeyListModel::rowCount(const QModelIndex &pidx) const
 
     // toplevel item:
     if (!pidx.isValid()) {
-        return mTopLevels.size();
+        return mTopLevels.size() + mGroups.size();
     }
 
     if (pidx.column() != 0) {
@@ -589,13 +903,15 @@ QModelIndex HierarchicalKeyListModel::index(int row, int col, const QModelIndex
 {
 
     if (row < 0 || col < 0 || col >= NumColumns) {
-        return QModelIndex();
+        return {};
     }
 
     // toplevel item:
     if (!pidx.isValid()) {
         if (static_cast<unsigned>(row) < mTopLevels.size()) {
             return index(mTopLevels[row], col);
+        } else if (static_cast<unsigned>(row) < mTopLevels.size() + mGroups.size()) {
+            return index(mGroups[row - mTopLevels.size()], col);
         } else {
             return QModelIndex();
         }
@@ -618,11 +934,11 @@ QModelIndex HierarchicalKeyListModel::parent(const QModelIndex &idx) const
 {
     const Key key = this->key(idx);
     if (key.isNull() || key.isRoot()) {
-        return QModelIndex();
+        return {};
     }
     const std::vector<Key>::const_iterator it
-        = qBinaryFind(mKeysByFingerprint.begin(), mKeysByFingerprint.end(),
-                      cleanChainID(key), _detail::ByFingerprint<std::less>());
+        = Kleo::binary_find(mKeysByFingerprint.begin(), mKeysByFingerprint.end(),
+                            cleanChainID(key), _detail::ByFingerprint<std::less>());
     return it != mKeysByFingerprint.end() ? index(*it) : QModelIndex();
 }
 
@@ -656,7 +972,7 @@ QModelIndex HierarchicalKeyListModel::doMapFromKey(const Key &key, int col) cons
 {
 
     if (key.isNull()) {
-        return QModelIndex();
+        return {};
     }
 
     const char *issuer_fpr = cleanChainID(key);
@@ -698,14 +1014,21 @@ void HierarchicalKeyListModel::addKeyWithParent(const char *issuer_fpr, const Ke
     if (it != subjects.end() && qstricmp(it->primaryFingerprint(), key.primaryFingerprint()) == 0) {
         // exists -> replace
         *it = key;
-        Q_EMIT dataChanged(createIndex(row, 0, const_cast<char *>(issuer_fpr)), createIndex(row, NumColumns - 1, const_cast<char *>(issuer_fpr)));
+        if (!modelResetInProgress()) {
+            Q_EMIT dataChanged(createIndex(row, 0, const_cast<char *>(issuer_fpr)), createIndex(row, NumColumns - 1, const_cast<char *>(issuer_fpr)));
+        }
     } else {
         // doesn't exist -> insert
-        const std::vector<Key>::const_iterator pos = qBinaryFind(mKeysByFingerprint.begin(), mKeysByFingerprint.end(), issuer_fpr, _detail::ByFingerprint<std::less>());
+        const std::vector<Key>::const_iterator pos = Kleo::binary_find(mKeysByFingerprint.begin(), mKeysByFingerprint.end(),
+                                                                       issuer_fpr, _detail::ByFingerprint<std::less>());
         Q_ASSERT(pos != mKeysByFingerprint.end());
-        beginInsertRows(index(*pos), row, row);
+        if (!modelResetInProgress()) {
+            beginInsertRows(index(*pos), row, row);
+        }
         subjects.insert(it, key);
-        endInsertRows();
+        if (!modelResetInProgress()) {
+            endInsertRows();
+        }
     }
 }
 
@@ -742,12 +1065,18 @@ void HierarchicalKeyListModel::addTopLevelKey(const Key &key)
     if (it != mTopLevels.end() && qstricmp(it->primaryFingerprint(), key.primaryFingerprint()) == 0) {
         // exists -> replace
         *it = key;
-        Q_EMIT dataChanged(createIndex(row, 0), createIndex(row, NumColumns - 1));
+        if (!modelResetInProgress()) {
+            Q_EMIT dataChanged(createIndex(row, 0), createIndex(row, NumColumns - 1));
+        }
     } else {
         // doesn't exist -> insert
-        beginInsertRows(QModelIndex(), row, row);
+        if (!modelResetInProgress()) {
+            beginInsertRows(QModelIndex(), row, row);
+        }
         mTopLevels.insert(it, key);
-        endInsertRows();
+        if (!modelResetInProgress()) {
+            endInsertRows();
+        }
     }
 
 }
@@ -765,7 +1094,7 @@ static std::vector<Key> topological_sort(const std::vector<Key> &keys)
             continue;
         }
         const std::vector<Key>::const_iterator it
-            = qBinaryFind(keys.begin(), keys.end(), issuer_fpr, _detail::ByFingerprint<std::less>());
+            = Kleo::binary_find(keys.begin(), keys.end(), issuer_fpr, _detail::ByFingerprint<std::less>());
         if (it == keys.end()) {
             continue;
         }
@@ -780,7 +1109,7 @@ static std::vector<Key> topological_sort(const std::vector<Key> &keys)
 
     std::vector<Key> result;
     result.reserve(keys.size());
-    for (int i : qAsConst(order)) {
+    for (int i : std::as_const(order)) {
         result.push_back(keys[i]);
     }
     return result;
@@ -806,7 +1135,8 @@ QList<QModelIndex> HierarchicalKeyListModel::doAddKeys(const std::vector<Key> &k
 
     std::set<Key, _detail::ByFingerprint<std::less> > changedParents;
 
-    Q_FOREACH (const Key &key, topological_sort(keys)) {
+    const auto topologicalSortedList = topological_sort(keys);
+    for (const Key &key : topologicalSortedList) {
 
         // check to see whether this key is a parent for a previously parent-less group:
         const char *const fpr = key.primaryFingerprint();
@@ -814,7 +1144,7 @@ QList<QModelIndex> HierarchicalKeyListModel::doAddKeys(const std::vector<Key> &k
             continue;
         }
 
-        const bool keyAlreadyExisted = qBinaryFind(oldKeys.begin(), oldKeys.end(), key, _detail::ByFingerprint<std::less>()) != oldKeys.end();
+        const bool keyAlreadyExisted = std::binary_search(oldKeys.begin(), oldKeys.end(), key, _detail::ByFingerprint<std::less>());
 
         const Map::iterator it = mKeysByNonExistingParent.find(fpr);
         const std::vector<Key> children = it != mKeysByNonExistingParent.end() ? it->second : std::vector<Key>();
@@ -825,22 +1155,26 @@ QList<QModelIndex> HierarchicalKeyListModel::doAddKeys(const std::vector<Key> &k
         // Step 1: For new keys, remove children from toplevel:
 
         if (!keyAlreadyExisted) {
-            std::vector<Key>::iterator last = mTopLevels.begin();
-            std::vector<Key>::iterator lastFP = mKeysByFingerprint.begin();
+            auto last = mTopLevels.begin();
+            auto lastFP = mKeysByFingerprint.begin();
 
-            for (const Key &k : qAsConst(children)) {
-                last = qBinaryFind(last, mTopLevels.end(), k, _detail::ByFingerprint<std::less>());
+            for (const Key &k : std::as_const(children)) {
+                last = Kleo::binary_find(last, mTopLevels.end(), k, _detail::ByFingerprint<std::less>());
                 Q_ASSERT(last != mTopLevels.end());
                 const int row = std::distance(mTopLevels.begin(), last);
 
-                lastFP = qBinaryFind(lastFP, mKeysByFingerprint.end(), k, _detail::ByFingerprint<std::less>());
+                lastFP = Kleo::binary_find(lastFP, mKeysByFingerprint.end(), k, _detail::ByFingerprint<std::less>());
                 Q_ASSERT(lastFP != mKeysByFingerprint.end());
 
                 Q_EMIT rowAboutToBeMoved(QModelIndex(), row);
-                beginRemoveRows(QModelIndex(), row, row);
+                if (!modelResetInProgress()) {
+                    beginRemoveRows(QModelIndex(), row, row);
+                }
                 last = mTopLevels.erase(last);
                 lastFP = mKeysByFingerprint.erase(lastFP);
-                endRemoveRows();
+                if (!modelResetInProgress()) {
+                    endRemoveRows();
+                }
             }
         }
         // Step 2: add/update key
@@ -881,10 +1215,12 @@ QList<QModelIndex> HierarchicalKeyListModel::doAddKeys(const std::vector<Key> &k
     }
     //Q_EMIT dataChanged for all parents with new children. This triggers KeyListSortFilterProxyModel to
     //show a parent node if it just got children matching the proxy's filter
-    for (const Key &i : qAsConst(changedParents)) {
-        const QModelIndex idx = index(i);
-        if (idx.isValid()) {
-            Q_EMIT dataChanged(idx.sibling(idx.row(), 0), idx.sibling(idx.row(), NumColumns - 1));
+    if (!modelResetInProgress()) {
+        for (const Key &i : std::as_const(changedParents)) {
+            const QModelIndex idx = index(i);
+            if (idx.isValid()) {
+                Q_EMIT dataChanged(idx.sibling(idx.row(), 0), idx.sibling(idx.row(), NumColumns - 1));
+            }
         }
     }
     return indexes(keys);
@@ -901,34 +1237,37 @@ void HierarchicalKeyListModel::doRemoveKey(const Key &key)
     if (mKeysByExistingParent.find(fpr) != mKeysByExistingParent.end()) {
         //handle non-leave nodes:
         std::vector<Key> keys = mKeysByFingerprint;
-        const std::vector<Key>::iterator it = qBinaryFind(keys.begin(), keys.end(),
-                                              key, _detail::ByFingerprint<std::less>());
+        const std::vector<Key>::iterator it = Kleo::binary_find(keys.begin(), keys.end(),
+                                                                key, _detail::ByFingerprint<std::less>());
         if (it == keys.end()) {
             return;
         }
         keys.erase(it);
         // FIXME for simplicity, we just clear the model and re-add all keys minus the removed one. This is suboptimal,
         // but acceptable given that deletion of non-leave nodes is rather rare.
-        clear();
+        clear(Keys);
         addKeys(keys);
         return;
     }
 
     //handle leave nodes:
 
-    const std::vector<Key>::iterator it = qBinaryFind(mKeysByFingerprint.begin(), mKeysByFingerprint.end(),
-                                          key, _detail::ByFingerprint<std::less>());
+    const std::vector<Key>::iterator it = Kleo::binary_find(mKeysByFingerprint.begin(), mKeysByFingerprint.end(),
+                                                            key, _detail::ByFingerprint<std::less>());
 
     Q_ASSERT(it != mKeysByFingerprint.end());
     Q_ASSERT(mKeysByNonExistingParent.find(fpr) == mKeysByNonExistingParent.end());
     Q_ASSERT(mKeysByExistingParent.find(fpr) == mKeysByExistingParent.end());
 
-    beginRemoveRows(parent(idx), idx.row(), idx.row());
+    if (!modelResetInProgress()) {
+        beginRemoveRows(parent(idx), idx.row(), idx.row());
+    }
     mKeysByFingerprint.erase(it);
 
     const char *const issuer_fpr = cleanChainID(key);
 
-    const std::vector<Key>::iterator tlIt = qBinaryFind(mTopLevels.begin(), mTopLevels.end(), key, _detail::ByFingerprint<std::less>());
+    const std::vector<Key>::iterator tlIt = Kleo::binary_find(mTopLevels.begin(), mTopLevels.end(),
+                                                              key, _detail::ByFingerprint<std::less>());
     if (tlIt != mTopLevels.end()) {
         mTopLevels.erase(tlIt);
     }
@@ -936,7 +1275,8 @@ void HierarchicalKeyListModel::doRemoveKey(const Key &key)
     if (issuer_fpr && *issuer_fpr) {
         const Map::iterator nexIt = mKeysByNonExistingParent.find(issuer_fpr);
         if (nexIt != mKeysByNonExistingParent.end()) {
-            const std::vector<Key>::iterator eit = qBinaryFind(nexIt->second.begin(), nexIt->second.end(), key, _detail::ByFingerprint<std::less>());
+            const std::vector<Key>::iterator eit = Kleo::binary_find(nexIt->second.begin(), nexIt->second.end(),
+                                                                     key, _detail::ByFingerprint<std::less>());
             if (eit != nexIt->second.end()) {
                 nexIt->second.erase(eit);
             }
@@ -947,7 +1287,8 @@ void HierarchicalKeyListModel::doRemoveKey(const Key &key)
 
         const Map::iterator exIt = mKeysByExistingParent.find(issuer_fpr);
         if (exIt != mKeysByExistingParent.end()) {
-            const std::vector<Key>::iterator eit = qBinaryFind(exIt->second.begin(), exIt->second.end(), key, _detail::ByFingerprint<std::less>());
+            const std::vector<Key>::iterator eit = Kleo::binary_find(exIt->second.begin(), exIt->second.end(),
+                                                                     key, _detail::ByFingerprint<std::less>());
             if (eit != exIt->second.end()) {
                 exIt->second.erase(eit);
             }
@@ -956,19 +1297,117 @@ void HierarchicalKeyListModel::doRemoveKey(const Key &key)
             }
         }
     }
-    endRemoveRows();
+    if (!modelResetInProgress()) {
+        endRemoveRows();
+    }
 }
 
-void AbstractKeyListModel::useKeyCache(bool value, bool secretOnly)
+KeyGroup HierarchicalKeyListModel::doMapToGroup(const QModelIndex &idx) const
 {
-    d->m_secretOnly = secretOnly;
-    d->m_useKeyCache = value;
-    if (value) {
-        setKeys(d->m_secretOnly ? KeyCache::instance()->secretKeys() : KeyCache::instance()->keys());
-    } else {
-        setKeys(std::vector<Key>());
+    Q_ASSERT(idx.isValid());
+    if (idx.parent().isValid()) {
+        // groups are always top-level
+        return KeyGroup();
     }
-    connect(KeyCache::instance().get(), &KeyCache::keysMayHaveChanged, this, [this] { if (d->m_useKeyCache) { setKeys(d->m_secretOnly ? KeyCache::instance()->secretKeys() : KeyCache::instance()->keys()); } });
+
+    if (static_cast<unsigned>(idx.row()) >= mTopLevels.size()
+            && static_cast<unsigned>(idx.row()) < mTopLevels.size() + mGroups.size()
+            && idx.column() < NumColumns) {
+        return mGroups[ idx.row() - mTopLevels.size() ];
+    } else {
+        return KeyGroup();
+    }
+}
+
+QModelIndex HierarchicalKeyListModel::doMapFromGroup(const KeyGroup &group, int column) const
+{
+    Q_ASSERT(!group.isNull());
+    const auto it = std::find_if(mGroups.cbegin(), mGroups.cend(),
+                                 [group](const KeyGroup &g) {
+                                     return g.source() == group.source() && g.id() == group.id();
+                                 });
+    if (it == mGroups.cend()) {
+        return QModelIndex();
+    } else {
+        return createIndex(it - mGroups.cbegin() + mTopLevels.size(), column);
+    }
+}
+
+void HierarchicalKeyListModel::doSetGroups(const std::vector<KeyGroup> &groups)
+{
+    Q_ASSERT(mGroups.empty());  // ensure that groups have been cleared
+    const int first = mTopLevels.size();
+    const int last = first + groups.size() - 1;
+    if (!modelResetInProgress()) {
+        beginInsertRows(QModelIndex(), first, last);
+    }
+    mGroups = groups;
+    if (!modelResetInProgress()) {
+        endInsertRows();
+    }
+}
+
+QModelIndex HierarchicalKeyListModel::doAddGroup(const KeyGroup &group)
+{
+    const int newRow = lastGroupRow() + 1;
+    if (!modelResetInProgress()) {
+        beginInsertRows(QModelIndex(), newRow, newRow);
+    }
+    mGroups.push_back(group);
+    if (!modelResetInProgress()) {
+        endInsertRows();
+    }
+    return createIndex(newRow, 0);
+}
+
+bool HierarchicalKeyListModel::doSetGroupData(const QModelIndex &index, const KeyGroup &group)
+{
+    if (group.isNull()) {
+        return false;
+    }
+    const int groupIndex = this->groupIndex(index);
+    if (groupIndex == -1) {
+        return false;
+    }
+    mGroups[groupIndex] = group;
+    if (!modelResetInProgress()) {
+        Q_EMIT dataChanged(createIndex(index.row(), 0), createIndex(index.row(), NumColumns - 1));
+    }
+    return true;
+}
+
+bool HierarchicalKeyListModel::doRemoveGroup(const KeyGroup &group)
+{
+    const QModelIndex modelIndex = doMapFromGroup(group, 0);
+    if (!modelIndex.isValid()) {
+        return false;
+    }
+    const int groupIndex = this->groupIndex(modelIndex);
+    Q_ASSERT(groupIndex != -1);
+    if (groupIndex == -1) {
+        return false;
+    }
+    if (!modelResetInProgress()) {
+        beginRemoveRows(QModelIndex(), modelIndex.row(), modelIndex.row());
+    }
+    mGroups.erase(mGroups.begin() + groupIndex);
+    if (!modelResetInProgress()) {
+        endRemoveRows();
+    }
+    return true;
+}
+
+void AbstractKeyListModel::useKeyCache(bool value, KeyList::Options options)
+{
+    d->m_keyListOptions = options;
+    d->m_useKeyCache = value;
+    if (!d->m_useKeyCache) {
+        clear(All);
+    } else {
+        d->updateFromKeyCache();
+    }
+    connect(KeyCache::instance().get(), &KeyCache::keysMayHaveChanged,
+            this, [this] { d->updateFromKeyCache(); });
 }
 
 // static
