@@ -2,38 +2,17 @@
     models/keylistsortfilterproxymodel.cpp
 
     This file is part of Kleopatra, the KDE keymanager
-    Copyright (c) 2007 Klarälvdalens Datakonsult AB
+    SPDX-FileCopyrightText: 2007 Klarälvdalens Datakonsult AB
 
-    Kleopatra is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    Kleopatra is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
-
-    In addition, as a special exception, the copyright holders give
-    permission to link the code of this program with any edition of
-    the Qt library by Trolltech AS, Norway (or with modified versions
-    of Qt that use the same license as Qt), and distribute linked
-    combinations including the two.  You must obey the GNU General
-    Public License in all respects for all of the code used other than
-    Qt.  If you modify this file, you may extend this exception to
-    your version of the file, but you are not obligated to do so.  If
-    you do not wish to do so, delete this exception statement from
-    your version.
+    SPDX-License-Identifier: GPL-2.0-or-later
 */
 
 #include "keylistsortfilterproxymodel.h"
 
+#include "keylist.h"
 #include "keylistmodel.h"
 #include "kleo/keyfilter.h"
+#include "kleo/keygroup.h"
 #include "kleo/stl_util.h"
 
 #include <libkleo_debug.h>
@@ -57,7 +36,7 @@ AbstractKeyListSortFilterProxyModel::AbstractKeyListSortFilterProxyModel(QObject
 AbstractKeyListSortFilterProxyModel::AbstractKeyListSortFilterProxyModel(const AbstractKeyListSortFilterProxyModel &other)
     : QSortFilterProxyModel(), KeyListModelInterface()
 {
-    Q_UNUSED(other);
+    Q_UNUSED(other)
     init();
 }
 
@@ -97,13 +76,20 @@ std::vector<Key> AbstractKeyListSortFilterProxyModel::keys(const QList<QModelInd
     return klmi->keys(mapped);
 }
 
+KeyGroup AbstractKeyListSortFilterProxyModel::group(const QModelIndex &idx) const
+{
+    if (const KeyListModelInterface *const klmi = dynamic_cast<KeyListModelInterface *>(sourceModel())) {
+        return klmi->group(mapToSource(idx));
+    }
+    return KeyGroup();
+}
+
 QModelIndex AbstractKeyListSortFilterProxyModel::index(const Key &key) const
 {
     if (const KeyListModelInterface *const klmi = dynamic_cast<KeyListModelInterface *>(sourceModel())) {
         return mapFromSource(klmi->index(key));
-    } else {
-        return QModelIndex();
     }
+    return {};
 }
 
 QList<QModelIndex> AbstractKeyListSortFilterProxyModel::indexes(const std::vector<Key> &keys) const
@@ -118,9 +104,16 @@ QList<QModelIndex> AbstractKeyListSortFilterProxyModel::indexes(const std::vecto
                            return mapFromSource(idx);
                        });
         return mapped;
-    } else {
-        return QList<QModelIndex>();
     }
+    return QList<QModelIndex>();
+}
+
+QModelIndex AbstractKeyListSortFilterProxyModel::index(const Kleo::KeyGroup& group) const
+{
+    if (const KeyListModelInterface *const klmi = dynamic_cast<KeyListModelInterface *>(sourceModel())) {
+        return mapFromSource(klmi->index(group));
+    }
+    return {};
 }
 
 class KeyListSortFilterProxyModel::Private
@@ -170,15 +163,15 @@ void KeyListSortFilterProxyModel::setKeyFilter(const std::shared_ptr<const KeyFi
 
 bool KeyListSortFilterProxyModel::filterAcceptsRow(int source_row, const QModelIndex &source_parent) const
 {
-
     //
     // 0. Keep parents of matching children:
     //
     const QModelIndex index = sourceModel()->index(source_row, 0, source_parent);
-    for (int i = 0, end = sourceModel()->rowCount(index); i != end; ++i)
+    for (int i = 0, end = sourceModel()->rowCount(index); i != end; ++i) {
         if (filterAcceptsRow(i, index)) {
             return true;
         }
+    }
 
     //
     // 1. Check filterRegExp
@@ -186,11 +179,13 @@ bool KeyListSortFilterProxyModel::filterAcceptsRow(int source_row, const QModelI
     const int role = filterRole();
     const int col = filterKeyColumn();
     const QRegExp rx = filterRegExp();
-    const QModelIndex nameIndex = sourceModel()->index(source_row, PrettyName, source_parent);
+    const QModelIndex nameIndex = sourceModel()->index(source_row, KeyList::PrettyName, source_parent);
 
     const KeyListModelInterface *const klm = dynamic_cast<KeyListModelInterface *>(sourceModel());
     Q_ASSERT(klm);
     const Key key = klm->key(nameIndex);
+    const KeyGroup group = klm->group(nameIndex);
+    Q_ASSERT(!key.isNull() || !group.isNull());
 
     if (col) {
         const QModelIndex colIdx = sourceModel()->index(source_row, col, source_parent);
@@ -198,7 +193,7 @@ bool KeyListSortFilterProxyModel::filterAcceptsRow(int source_row, const QModelI
         if (!content.contains(rx)) {
             return false;
         }
-    } else {
+    } else if (!key.isNull()) {
         // By default match against the full uid data (name / email / comment / dn)
         bool match = false;
         for (const auto &uid: key.userIDs()) {
@@ -211,7 +206,7 @@ bool KeyListSortFilterProxyModel::filterAcceptsRow(int source_row, const QModelI
             // Also match against remarks (search tags)
             const auto alm = dynamic_cast<AbstractKeyListModel *>(sourceModel());
             if (alm) {
-                const auto remarks = alm->data(alm->index(key, KeyListModelInterface::Remarks));
+                const auto remarks = alm->data(alm->index(key, KeyList::Remarks));
                 if (!remarks.isNull() && remarks.toString().contains(rx)) {
                     match = true;
                     break;
@@ -222,12 +217,18 @@ bool KeyListSortFilterProxyModel::filterAcceptsRow(int source_row, const QModelI
         if (!match) {
             return false;
         }
+    } else if (!group.isNull()) {
+        if (!group.name().contains(rx)) {
+            return false;
+        }
+    } else {
+        return false;
     }
 
     //
-    // 2. Check that key filters match (if any are defined)
+    // 2. For keys check that key filters match (if any are defined)
     //
-    if (d->keyFilter) {   // avoid artifacts when no filters are defined
+    if (d->keyFilter && !key.isNull()) {   // avoid artifacts when no filters are defined
         return d->keyFilter->matches(key, KeyFilter::Filtering);
     }
 
