@@ -2,40 +2,16 @@
     dnattributeorderconfigwidget.cpp
 
     This file is part of libkleopatra, the KDE keymanagement library
-    Copyright (c) 2004 Klarävdalens Datakonsult AB
+    SPDX-FileCopyrightText: 2004 Klarälvdalens Datakonsult AB
 
-    Libkleopatra is free software; you can redistribute it and/or
-    modify it under the terms of the GNU General Public License as
-    published by the Free Software Foundation; either version 2 of the
-    License, or (at your option) any later version.
-
-    Libkleopatra is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
-
-    In addition, as a special exception, the copyright holders give
-    permission to link the code of this program with any edition of
-    the Qt library by Trolltech AS, Norway (or with modified versions
-    of Qt that use the same license as Qt), and distribute linked
-    combinations including the two.  You must obey the GNU General
-    Public License in all respects for all of the code used other than
-    Qt.  If you modify this file, you may extend this exception to
-    your version of the file, but you are not obligated to do so.  If
-    you do not wish to do so, delete this exception statement from
-    your version.
- */
+    SPDX-License-Identifier: GPL-2.0-or-later
+*/
 
 #include "dnattributeorderconfigwidget.h"
 #include "libkleo_debug.h"
 #include "libkleo/dn.h"
 
 #include <KLocalizedString>
-#include <KConfig>
 #include <QIcon>
 
 #include <QToolButton>
@@ -46,7 +22,7 @@
 #include <QHeaderView>
 
 
-class Kleo::DNAttributeOrderConfigWidget::Private
+class Kleo::DNAttributeOrderConfigWidget::DNAttributeOrderConfigWidgetPrivate
 {
 public:
     enum { UUp = 0, Up = 1, Left = 2, Right = 3, Down = 4, DDown = 5 };
@@ -74,12 +50,12 @@ static void prepare(QTreeWidget *lv)
 #endif
 
 Kleo::DNAttributeOrderConfigWidget::DNAttributeOrderConfigWidget(DNAttributeMapper *mapper, QWidget *parent, Qt::WindowFlags f)
-    : QWidget(parent, f), d(new Private)
+    : QWidget(parent, f), d(new DNAttributeOrderConfigWidgetPrivate)
 {
     Q_ASSERT(mapper);
     d->mapper = mapper;
 
-    QGridLayout *glay = new QGridLayout(this);
+    auto glay = new QGridLayout(this);
     glay->setContentsMargins(0, 0, 0, 0);
     glay->setColumnStretch(0, 1);
     glay->setColumnStretch(2, 1);
@@ -113,7 +89,7 @@ Kleo::DNAttributeOrderConfigWidget::DNAttributeOrderConfigWidget(DNAttributeMapp
 
     // the up/down/left/right arrow cross:
 
-    QGridLayout *xlay = new QGridLayout();
+    auto xlay = new QGridLayout();
     xlay->setSpacing(0);
     xlay->setObjectName(QStringLiteral("xlay"));
     xlay->setAlignment(Qt::AlignCenter);
@@ -146,10 +122,7 @@ Kleo::DNAttributeOrderConfigWidget::DNAttributeOrderConfigWidget(DNAttributeMapp
     glay->addLayout(xlay, row, 1);
 }
 
-Kleo::DNAttributeOrderConfigWidget::~DNAttributeOrderConfigWidget()
-{
-    delete d;
-}
+Kleo::DNAttributeOrderConfigWidget::~DNAttributeOrderConfigWidget() = default;
 
 void Kleo::DNAttributeOrderConfigWidget::load()
 {
@@ -183,7 +156,7 @@ void Kleo::DNAttributeOrderConfigWidget::load()
     const QStringList::const_iterator end(all.end());
     for (QStringList::const_iterator it = all.begin(); it != end; ++it) {
         if (!order.contains(*it)) {
-            QTreeWidgetItem *item = new QTreeWidgetItem(d->availableLV);
+            auto item = new QTreeWidgetItem(d->availableLV);
             item->setText(0, *it);
             item->setText(1, d->mapper->name2label(*it));
         }
@@ -223,7 +196,7 @@ void Kleo::DNAttributeOrderConfigWidget::defaults()
 
 void Kleo::DNAttributeOrderConfigWidget::slotAvailableSelectionChanged(QTreeWidgetItem *item)
 {
-    d->navTB[Private::Right]->setEnabled(item);
+    d->navTB[DNAttributeOrderConfigWidgetPrivate::Right]->setEnabled(item);
 }
 
 void Kleo::DNAttributeOrderConfigWidget::slotCurrentOrderSelectionChanged(QTreeWidgetItem *item)
@@ -234,11 +207,11 @@ void Kleo::DNAttributeOrderConfigWidget::slotCurrentOrderSelectionChanged(QTreeW
 void Kleo::DNAttributeOrderConfigWidget::enableDisableButtons(QTreeWidgetItem *item)
 {
 #ifndef QT_NO_TREEWIDGET
-    d->navTB[Private::UUp  ]->setEnabled(item && d->currentLV->itemAbove(item));
-    d->navTB[Private::Up   ]->setEnabled(item && d->currentLV->itemAbove(item));
-    d->navTB[Private::Left ]->setEnabled(item);
-    d->navTB[Private::Down ]->setEnabled(item && d->currentLV->itemBelow(item));
-    d->navTB[Private::DDown]->setEnabled(item && d->currentLV->itemBelow(item));
+    d->navTB[DNAttributeOrderConfigWidgetPrivate::UUp  ]->setEnabled(item && d->currentLV->itemAbove(item));
+    d->navTB[DNAttributeOrderConfigWidgetPrivate::Up   ]->setEnabled(item && d->currentLV->itemAbove(item));
+    d->navTB[DNAttributeOrderConfigWidgetPrivate::Left ]->setEnabled(item);
+    d->navTB[DNAttributeOrderConfigWidgetPrivate::Down ]->setEnabled(item && d->currentLV->itemBelow(item));
+    d->navTB[DNAttributeOrderConfigWidgetPrivate::DDown]->setEnabled(item && d->currentLV->itemBelow(item));
 #endif
 }
 
@@ -365,7 +338,7 @@ void Kleo::DNAttributeOrderConfigWidget::slotRightButtonClicked()
     d->currentLV->insertTopLevelItem(newRightIndex, left);
     left->setSelected(true);
     enableDisableButtons(left);
-    d->navTB[Private::Right]->setEnabled(next);
+    d->navTB[DNAttributeOrderConfigWidgetPrivate::Right]->setEnabled(next);
     if (next) {
         next->setSelected(true);
     }
