@@ -7,8 +7,7 @@
     SPDX-License-Identifier: GPL-2.0-or-later
 */
 
-#ifndef __KLEOPATRA_UI_FILENAMEREQUESTER_H__
-#define __KLEOPATRA_UI_FILENAMEREQUESTER_H__
+#pragma once
 
 #include "kleo_export.h"
 
@@ -27,7 +26,7 @@ class KLEO_EXPORT FileNameRequester : public QWidget
 public:
     explicit FileNameRequester(QWidget *parent = nullptr);
     explicit FileNameRequester(QDir::Filters filter, QWidget *parent = nullptr);
-    ~FileNameRequester();
+    ~FileNameRequester() override;
 
     void setFileName(const QString &name);
     QString fileName() const;
@@ -44,14 +43,16 @@ public:
 Q_SIGNALS:
     void fileNameChanged(const QString &filename);
 
+protected:
+    bool event(QEvent *event) override;
+
 private:
     virtual QString requestFileName();
 
 private:
-    class Private;
-    Private *d;
+    class FileNameRequesterPrivate;
+    std::unique_ptr<FileNameRequesterPrivate> const d;
 };
 
 }
 
-#endif /* __KLEOPATRA_UI_FILENAMEREQUESTER_H__ */
