@@ -2,40 +2,16 @@
     keyselectiondialog.h
 
     This file is part of libkleopatra, the KDE keymanagement library
-    Copyright (c) 2004 Klarälvdalens Datakonsult AB
+    SPDX-FileCopyrightText: 2004 Klarälvdalens Datakonsult AB
 
     Based on kpgpui.h
-    Copyright (C) 2001,2002 the KPGP authors
+    SPDX-FileCopyrightText: 2001, 2002 the KPGP authors
     See file libkdenetwork/AUTHORS.kpgp for details
 
-    Libkleopatra is free software; you can redistribute it and/or
-    modify it under the terms of the GNU General Public License as
-    published by the Free Software Foundation; either version 2 of the
-    License, or (at your option) any later version.
-
-    Libkleopatra is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
-
-    In addition, as a special exception, the copyright holders give
-    permission to link the code of this program with any edition of
-    the Qt library by Trolltech AS, Norway (or with modified versions
-    of Qt that use the same license as Qt), and distribute linked
-    combinations including the two.  You must obey the GNU General
-    Public License in all respects for all of the code used other than
-    Qt.  If you modify this file, you may extend this exception to
-    your version of the file, but you are not obligated to do so.  If
-    you do not wish to do so, delete this exception statement from
-    your version.
+    SPDX-License-Identifier: GPL-2.0-or-later
 */
 
-#ifndef __KLEO_UI_KEYSELECTIONDIALOG_H__
-#define __KLEO_UI_KEYSELECTIONDIALOG_H__
+#pragma once
 
 #include "kleo_export.h"
 
@@ -43,17 +19,16 @@
 #include <qgpgme/protocol.h>
 
 #include <QDialog>
-
 #include <QPixmap>
 
 #include <vector>
 
-class QVBoxLayout;
 class QCheckBox;
-class QPixmap;
-class QTimer;
-class QRegExp;
+class QLabel;
 class QPoint;
+class QRegExp;
+class QTimer;
+class QVBoxLayout;
 
 namespace Kleo
 {
@@ -73,6 +48,13 @@ class KLEO_EXPORT KeySelectionDialog : public QDialog
 {
     Q_OBJECT
 public:
+    enum Option {
+        RereadKeys = 0x01,
+        ExternalCertificateManager = 0x02,
+        ExtendedSelection = 0x04,
+        RememberChoice = 0x08
+    };
+    Q_DECLARE_FLAGS(Options, Option)
 
     enum KeyUsage {
         PublicKeys = 1,
@@ -90,6 +72,8 @@ public:
         ValidTrustedEncryptionKeys = AllKeys | EncryptionKeys | ValidKeys | TrustedKeys
     };
 
+    explicit KeySelectionDialog(QWidget *parent = nullptr, Options options = Options());
+
     KeySelectionDialog(const QString &title,
                        const QString &text,
                        const std::vector<GpgME::Key> &selectedKeys = std::vector<GpgME::Key>(),
@@ -115,7 +99,11 @@ public:
                        bool rememberChoice = false,
                        QWidget *parent = nullptr,
                        bool modal = true);
-    ~KeySelectionDialog();
+    ~KeySelectionDialog() override;
+
+    void setText(const QString &text);
+
+    void setKeys(const std::vector<GpgME::Key> &keys);
 
     /** Returns the key ID of the selected key in single selection mode.
         Otherwise it returns a null key. */
@@ -173,7 +161,6 @@ private:
     void filterByKeyIDOrUID(const QString &keyID);
     void filterByUID(const QString &uid);
     void showAllItems();
-    bool anyChildMatches(const Kleo::KeyListViewItem *item, QRegExp &rx) const;
 
     void connectSignals();
     void disconnectSignals();
@@ -181,14 +168,19 @@ private:
     void startKeyListJobForBackend(const QGpgME::Protocol *, const std::vector<GpgME::Key> &, bool);
     void startValidatingKeyListing();
 
+    void setUpUI(Options options, const QString &);
     void init(bool, bool, const QString &, const QString &);
 
 private:
     QVBoxLayout *mTopLayout = nullptr;
+    QLabel *mTextLabel = nullptr;
     Kleo::KeyListView *mKeyListView = nullptr;
+    Kleo::KeyListViewItem *mCurrentContextMenuItem = nullptr;
+    QCheckBox *mRememberCB = nullptr;
+    QPushButton *mOkButton = nullptr;
+
     const QGpgME::Protocol *mOpenPGPBackend = nullptr;
     const QGpgME::Protocol *mSMIMEBackend = nullptr;
-    QCheckBox *mRememberCB = nullptr;
     std::vector<GpgME::Key> mSelectedKeys, mKeysToCheck;
     unsigned int mKeyUsage;
     QTimer *mCheckSelectionTimer = nullptr;
@@ -196,11 +188,12 @@ private:
     // cross-eventloop temporaries:
     QString mSearchText;
     const QString mInitialQuery;
-    Kleo::KeyListViewItem *mCurrentContextMenuItem = nullptr;
-    int mTruncated, mListJobCount, mSavedOffsetY;
-    QPushButton *mOkButton = nullptr;
+    int mTruncated = 0,
+        mListJobCount = 0,
+        mSavedOffsetY = 0;
 };
 
 }
 
-#endif // __KLEO_UI_KEYSELECTIONDIALOG_H__
+Q_DECLARE_OPERATORS_FOR_FLAGS(Kleo::KeySelectionDialog::Options)
+
