@@ -2,44 +2,21 @@
     utils/classify.cpp
 
     This file is part of Kleopatra, the KDE keymanager
-    Copyright (c) 2007 Klarälvdalens Datakonsult AB
+    SPDX-FileCopyrightText: 2007 Klarälvdalens Datakonsult AB
 
-    Kleopatra is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    Kleopatra is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
-
-    In addition, as a special exception, the copyright holders give
-    permission to link the code of this program with any edition of
-    the Qt library by Trolltech AS, Norway (or with modified versions
-    of Qt that use the same license as Qt), and distribute linked
-    combinations including the two.  You must obey the GNU General
-    Public License in all respects for all of the code used other than
-    Qt.  If you modify this file, you may extend this exception to
-    your version of the file, but you are not obligated to do so.  If
-    you do not wish to do so, delete this exception statement from
-    your version.
+    SPDX-License-Identifier: GPL-2.0-or-later
 */
 
 #include "classify.h"
 
 #include "libkleo_debug.h"
 #include "kleo/checksumdefinition.h"
+#include "utils/algorithm.h"
 
 #include <QString>
 #include <QStringList>
 #include <QFile>
 #include <QFileInfo>
-#include <QtAlgorithms>
 #include <QByteArrayMatcher>
 #include <QMap>
 #include <QRegularExpression>
@@ -62,45 +39,45 @@ static const struct _classification {
     unsigned int classification;
 } classifications[] = {
     // ordered by extension
-    { "arl", CMS    | Binary  | CertificateRevocationList },
-    { "asc", OpenPGP |  Ascii  | OpaqueSignature | DetachedSignature | CipherText | AnyCertStoreType | ExamineContentHint },
-    { "cer", CMS    | Binary  | Certificate },
-    { "crl", CMS    | Binary  | CertificateRevocationList },
-    { "crt", CMS    | Binary  | Certificate },
-    { "der", CMS    | Binary  | Certificate | CertificateRevocationList },
-    { "gpg", OpenPGP | Binary  | OpaqueSignature | CipherText | AnyCertStoreType | ExamineContentHint},
-    { "p10", CMS    |  Ascii  | CertificateRequest },
-    { "p12", CMS    | Binary  | ExportedPSM },
-    { "p7c", CMS    | Binary  | Certificate  },
-    { "p7m", CMS    | AnyFormat | CipherText },
-    { "p7s", CMS    | AnyFormat | AnySignature },
-    { "pem", CMS    |  Ascii  | AnyType | ExamineContentHint },
-    { "pfx", CMS    | Binary  | Certificate },
-    { "pgp", OpenPGP | Binary  | OpaqueSignature | CipherText | AnyCertStoreType | ExamineContentHint},
-    { "sig", OpenPGP | AnyFormat | DetachedSignature },
+    { "arl", Kleo::Class::CMS    | Binary  | CertificateRevocationList },
+    { "asc", Kleo::Class::OpenPGP |  Ascii  | OpaqueSignature | DetachedSignature | CipherText | AnyCertStoreType | ExamineContentHint },
+    { "cer", Kleo::Class::CMS    | Binary  | Certificate },
+    { "crl", Kleo::Class::CMS    | Binary  | CertificateRevocationList },
+    { "crt", Kleo::Class::CMS    | Binary  | Certificate },
+    { "der", Kleo::Class::CMS    | Binary  | Certificate | CertificateRevocationList },
+    { "gpg", Kleo::Class::OpenPGP | Binary  | OpaqueSignature | CipherText | AnyCertStoreType | ExamineContentHint},
+    { "p10", Kleo::Class::CMS    |  Ascii  | CertificateRequest },
+    { "p12", Kleo::Class::CMS    | Binary  | ExportedPSM },
+    { "p7c", Kleo::Class::CMS    | Binary  | Certificate  },
+    { "p7m", Kleo::Class::CMS    | AnyFormat | CipherText },
+    { "p7s", Kleo::Class::CMS    | AnyFormat | AnySignature },
+    { "pem", Kleo::Class::CMS    |  Ascii  | AnyType | ExamineContentHint },
+    { "pfx", Kleo::Class::CMS    | Binary  | Certificate },
+    { "pgp", Kleo::Class::OpenPGP | Binary  | OpaqueSignature | CipherText | AnyCertStoreType | ExamineContentHint},
+    { "sig", Kleo::Class::OpenPGP | AnyFormat | DetachedSignature },
 };
 
 static const QMap<GpgME::Data::Type, unsigned int> gpgmeTypeMap {
-    { GpgME::Data::PGPSigned, OpenPGP | OpaqueSignature },
+    { GpgME::Data::PGPSigned, Kleo::Class::OpenPGP | OpaqueSignature },
     /* PGPOther might be just an unencrypted unsigned pgp message. Decrypt
      * would yield the plaintext anyway so for us this is CipherText. */
-    { GpgME::Data::PGPOther, OpenPGP | CipherText },
-    { GpgME::Data::PGPKey, OpenPGP | Certificate },
-    { GpgME::Data::CMSSigned, CMS | AnySignature },
-    { GpgME::Data::CMSEncrypted, CMS | CipherText },
+    { GpgME::Data::PGPOther, Kleo::Class::OpenPGP | CipherText },
+    { GpgME::Data::PGPKey, Kleo::Class::OpenPGP | Certificate },
+    { GpgME::Data::CMSSigned, Kleo::Class::CMS | AnySignature },
+    { GpgME::Data::CMSEncrypted, Kleo::Class::CMS | CipherText },
     /* See PGPOther */
-    { GpgME::Data::CMSOther, CMS | CipherText },
-    { GpgME::Data::X509Cert, CMS | Certificate},
-    { GpgME::Data::PKCS12, CMS | Binary | ExportedPSM },
-    { GpgME::Data::PGPEncrypted, OpenPGP | CipherText },
-    { GpgME::Data::PGPSignature, OpenPGP | DetachedSignature },
+    { GpgME::Data::CMSOther, Kleo::Class::CMS | CipherText },
+    { GpgME::Data::X509Cert, Kleo::Class::CMS | Certificate},
+    { GpgME::Data::PKCS12, Kleo::Class::CMS | Binary | ExportedPSM },
+    { GpgME::Data::PGPEncrypted, Kleo::Class::OpenPGP | CipherText },
+    { GpgME::Data::PGPSignature, Kleo::Class::OpenPGP | DetachedSignature },
 };
 
 static const unsigned int defaultClassification = NoClass;
 
 template <template <typename U> class Op>
 struct ByExtension {
-    typedef bool result_type;
+    using result_type = bool;
 
     template <typename T>
     bool operator()(const T &lhs, const T &rhs) const
@@ -139,7 +116,7 @@ static const struct _content_classification {
 
 template <template <typename U> class Op>
 struct ByContent {
-    typedef bool result_type;
+    using result_type = bool;
 
     const unsigned int N;
     explicit ByContent(unsigned int n) : N(n) {}
@@ -181,13 +158,14 @@ unsigned int Kleo::classify(const QStringList &fileNames)
 
 static unsigned int classifyExtension(const QFileInfo &fi)
 {
-    const _classification *const it = qBinaryFind(std::begin(classifications), std::end(classifications),
-                                      fi.suffix().toLatin1().constData(),
-                                      ByExtension<std::less>());
-    if (it != std::end(classifications))
+    const _classification *const it = Kleo::binary_find(std::begin(classifications), std::end(classifications),
+                                                        fi.suffix().toLatin1().constData(),
+                                                        ByExtension<std::less>());
+    if (it != std::end(classifications)) {
         if (!(it->classification & ExamineContentHint)) {
             return it->classification;
         }
+    }
 
     return it == std::end(classifications) ? defaultClassification
                                         : it->classification;
@@ -256,11 +234,12 @@ static unsigned int classifyContentInteral(const QByteArray &data)
     }
 
     const _content_classification *const cit
-        = qBinaryFind(std::begin(content_classifications), std::end(content_classifications),
-                      data.data() + pos, ByContent<std::less>(epos - pos));
+        = Kleo::binary_find(std::begin(content_classifications), std::end(content_classifications),
+                            data.data() + pos,
+                            ByContent<std::less>(epos - pos));
 
     if (cit != std::end(content_classifications)) {
-        return cit->classification | (pgp ? OpenPGP : CMS);
+        return cit->classification | (pgp ? Kleo::Class::OpenPGP : Kleo::Class::CMS);
     }
     return defaultClassification;
 }
@@ -291,37 +270,37 @@ unsigned int Kleo::classifyContent(const QByteArray &data)
 QString Kleo::printableClassification(unsigned int classification)
 {
     QStringList parts;
-    if (classification & CMS) {
+    if (classification & Kleo::Class::CMS) {
         parts.push_back(QStringLiteral("CMS"));
     }
-    if (classification & OpenPGP) {
+    if (classification & Kleo::Class::OpenPGP) {
         parts.push_back(QStringLiteral("OpenPGP"));
     }
-    if (classification & Binary) {
+    if (classification & Kleo::Class::Binary) {
         parts.push_back(QStringLiteral("Binary"));
     }
-    if (classification & Ascii) {
+    if (classification & Kleo::Class::Ascii) {
         parts.push_back(QStringLiteral("Ascii"));
     }
-    if (classification & DetachedSignature) {
+    if (classification & Kleo::Class::DetachedSignature) {
         parts.push_back(QStringLiteral("DetachedSignature"));
     }
-    if (classification & OpaqueSignature) {
+    if (classification & Kleo::Class::OpaqueSignature) {
         parts.push_back(QStringLiteral("OpaqueSignature"));
     }
-    if (classification & ClearsignedMessage) {
+    if (classification & Kleo::Class::ClearsignedMessage) {
         parts.push_back(QStringLiteral("ClearsignedMessage"));
     }
-    if (classification & CipherText) {
+    if (classification & Kleo::Class::CipherText) {
         parts.push_back(QStringLiteral("CipherText"));
     }
-    if (classification & Certificate) {
+    if (classification & Kleo::Class::Certificate) {
         parts.push_back(QStringLiteral("Certificate"));
     }
-    if (classification & ExportedPSM) {
+    if (classification & Kleo::Class::ExportedPSM) {
         parts.push_back(QStringLiteral("ExportedPSM"));
     }
-    if (classification & CertificateRequest) {
+    if (classification & Kleo::Class::CertificateRequest) {
         parts.push_back(QStringLiteral("CertificateRequest"));
     }
     return parts.join(QLatin1String(", "));
@@ -355,13 +334,14 @@ QString Kleo::findSignedData(const QString &signatureFileName)
 QStringList Kleo::findSignatures(const QString &signedDataFileName)
 {
     QStringList result;
-    for (unsigned int i = 0, end = sizeof(classifications) / sizeof(_classification); i < end; ++i)
+    for (unsigned int i = 0, end = sizeof(classifications) / sizeof(_classification); i < end; ++i) {
         if (classifications[i].classification & DetachedSignature) {
             const QString candidate = signedDataFileName + QLatin1Char('.') + QLatin1String(classifications[i].extension);
             if (QFile::exists(candidate)) {
                 result.push_back(candidate);
             }
         }
+    }
     return result;
 }
 
@@ -373,9 +353,9 @@ QString Kleo::outputFileName(const QString &inputFileName)
 {
     const QFileInfo fi(inputFileName);
 
-    if (qBinaryFind(std::begin(classifications), std::end(classifications),
-                    fi.suffix().toLatin1().constData(),
-                    ByExtension<std::less>()) == std::end(classifications)) {
+    if (!std::binary_search(std::begin(classifications), std::end(classifications),
+                            fi.suffix().toLatin1().constData(),
+                            ByExtension<std::less>())) {
         return inputFileName + QLatin1String(".out");
     } else {
         return chopped(inputFileName, 4);
@@ -389,14 +369,15 @@ QString Kleo::outputFileName(const QString &inputFileName)
 const char *Kleo::outputFileExtension(unsigned int classification, bool usePGPFileExt)
 {
 
-    if (classification & OpenPGP && usePGPFileExt) {
+    if (classification & Kleo::Class::OpenPGP && usePGPFileExt) {
         return "pgp";
     }
 
-    for (unsigned int i = 0; i < sizeof classifications / sizeof * classifications; ++i)
+    for (unsigned int i = 0; i < sizeof classifications / sizeof *classifications; ++i) {
         if ((classifications[i].classification & classification) == classification) {
             return classifications[i].extension;
         }
+    }
     return nullptr;
 }
 
@@ -415,9 +396,11 @@ bool Kleo::isChecksumFile(const QString &file)
         return false;
     }
     if (!initialized) {
-        Q_FOREACH (const std::shared_ptr<ChecksumDefinition> &cd, ChecksumDefinition::getChecksumDefinitions()) {
+        const auto getChecksumDefinitions = ChecksumDefinition::getChecksumDefinitions();
+        for (const std::shared_ptr<ChecksumDefinition> &cd : getChecksumDefinitions) {
             if (cd) {
-                Q_FOREACH (const QString &pattern, cd->patterns()) {
+                const auto patternsList = cd->patterns();
+                for (const QString &pattern : patternsList) {
 #ifdef Q_OS_WIN
                     patterns << QRegExp(pattern, Qt::CaseInsensitive);
 #else
@@ -430,7 +413,7 @@ bool Kleo::isChecksumFile(const QString &file)
     }
 
     const QString fileName = fi.fileName();
-    for (const QRegExp &pattern : qAsConst(patterns)) {
+    for (const QRegExp &pattern : std::as_const(patterns)) {
         if (pattern.exactMatch(fileName)) {
             return true;
         }
