@@ -1,38 +1,23 @@
 /* -*- mode: c++; c-basic-offset: 4; indent-tabs-mode: nil; -*-
-
     utils/formatting.cpp
 
     This file is part of Kleopatra, the KDE keymanager
-    Copyright (c) 2007 Klarälvdalens Datakonsult AB
+    SPDX-FileCopyrightText: 2007 Klarälvdalens Datakonsult AB
+    SPDX-FileCopyrightText: 2021 g10 Code GmbH
+    SPDX-FileContributor: Ingo Klöcker <dev@ingo-kloecker.de>
 
-    Kleopatra is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    Kleopatra is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
-
-    In addition, as a special exception, the copyright holders give
-    permission to link the code of this program with any edition of
-    the Qt library by Trolltech AS, Norway (or with modified versions
-    of Qt that use the same license as Qt), and distribute linked
-    combinations including the two.  You must obey the GNU General
-    Public License in all respects for all of the code used other than
-    Qt.  If you modify this file, you may extend this exception to
-    your version of the file, but you are not obligated to do so.  If
-    you do not wish to do so, delete this exception statement from
-    your version.
+    SPDX-License-Identifier: GPL-2.0-or-later
 */
 
+#include <config-libkleo.h>
+
 #include "formatting.h"
 #include "kleo/dn.h"
+#include "kleo/keyfiltermanager.h"
+#include "kleo/keygroup.h"
+
+#include "utils/cryptoconfig.h"
+#include "utils/gnupg.h"
 
 #include <gpgme++/key.h>
 #include <gpgme++/importresult.h>
@@ -44,7 +29,6 @@
 #include <KEmailAddress>
 
 #include <QString>
-#include <QStringList>
 #include <QDateTime>
 #include <QTextDocument> // for Qt::escape
 #include <QLocale>
@@ -208,7 +192,9 @@ QString Formatting::prettyEMail(const UserID::Signature &sig)
 
 QString Formatting::prettyEMail(const char *email_, const char *id)
 {
-    QString email, name, comment;
+    QString email;
+    QString name;
+    QString comment;
     if (email_ && KEmailAddress::splitAddress(QString::fromUtf8(email_),
                                               name, email, comment) == KEmailAddress::AddressOk) {
         return email;
@@ -226,7 +212,8 @@ namespace
 
 static QString protect_whitespace(QString s)
 {
-    static const QLatin1Char SP(' '), NBSP('\xA0');
+    static const QLatin1Char SP(' ');
+    static const QLatin1Char NBSP('\xA0');
     return s.replace(SP, NBSP);
 }
 
@@ -335,14 +322,14 @@ QString Formatting::toolTip(const Key &key, int flags)
 
     QString result;
     if (flags & Validity) {
-        if (key.protocol() == OpenPGP || (key.keyListMode() & Validate))
+        if (key.protocol() == OpenPGP || (key.keyListMode() & Validate)) {
             if (key.isRevoked()) {
                 result = make_red(i18n("Revoked"));
             } else if (key.isExpired()) {
                 result = make_red(i18n("Expired"));
             } else if (key.isDisabled()) {
                 result = i18n("Disabled");
-            } else {
+            } else if (key.keyListMode() & GpgME::Validate) {
                 unsigned int fullyTrusted = 0;
                 for (const auto &uid: key.userIDs()) {
                     if (uid.validity() >= UserID::Validity::Full) {
@@ -358,8 +345,10 @@ QString Formatting::toolTip(const Key &key, int flags)
                 } else {
                     result = i18np("One User-ID is not certified.", "%1 User-IDs are not certified.", key.numUserIDs() - fullyTrusted);
                 }
+            } else {
+                result = i18n("The validity cannot be checked at the moment.");
             }
-        else {
+        } else {
             result = i18n("The validity cannot be checked at the moment.");
         }
     }
@@ -378,15 +367,18 @@ QString Formatting::toolTip(const Key &key, int flags)
     }
     if (flags & UserIDs) {
         const std::vector<UserID> uids = key.userIDs();
-        if (!uids.empty())
+        if (!uids.empty()) {
             result += format_row(key.protocol() == CMS
                                  ? i18n("Subject")
                                  : i18n("User-ID"), prettyUserID(uids.front()));
-        if (uids.size() > 1)
-            for (std::vector<UserID>::const_iterator it = uids.begin() + 1, end = uids.end(); it != end; ++it)
+        }
+        if (uids.size() > 1) {
+            for (auto it = uids.begin() + 1, end = uids.end(); it != end; ++it) {
                 if (!it->isRevoked() && !it->isInvalid()) {
                     result += format_row(i18n("a.k.a."), prettyUserID(*it));
                 }
+            }
+        }
     }
     if (flags & ExpiryDates) {
         result += format_row(i18n("Created"), time_t2string(subkey.creationTime()));
@@ -465,6 +457,75 @@ QString Formatting::toolTip(const Key &key, int flags)
     return result;
 }
 
+namespace
+{
+template <typename Container>
+QString getValidityStatement(const Container &keys)
+{
+    const bool allKeysAreOpenPGP = std::all_of(keys.cbegin(), keys.cend(), [] (const Key &key) { return key.protocol() == OpenPGP; });
+    const bool allKeysAreValidated = std::all_of(keys.cbegin(), keys.cend(), [] (const Key &key) { return key.keyListMode() & Validate; });
+    if (allKeysAreOpenPGP || allKeysAreValidated) {
+        const bool someKeysAreBad = std::any_of(keys.cbegin(), keys.cend(), std::mem_fn(&Key::isBad));
+        if (someKeysAreBad) {
+            return i18n("Some keys are revoked, expired, disabled, or invalid.");
+        } else {
+            const bool allKeysAreFullyValid = std::all_of(keys.cbegin(), keys.cend(), &Formatting::uidsHaveFullValidity);
+            if (allKeysAreFullyValid) {
+                return i18n("All keys are certified.");
+            } else {
+                return i18n("Some keys are not certified.");
+            }
+        }
+    }
+    return i18n("The validity of the keys cannot be checked at the moment.");
+}
+}
+
+QString Formatting::toolTip(const KeyGroup &group, int flags)
+{
+    static const unsigned int maxNumKeysForTooltip = 20;
+
+    if (group.isNull()) {
+        return QString();
+    }
+
+    const KeyGroup::Keys &keys = group.keys();
+    if (keys.size() == 0) {
+        return i18nc("@info:tooltip", "This group does not contain any keys.");
+    }
+
+    const QString validity = (flags & Validity) ? getValidityStatement(keys) : QString();
+    if (flags == Validity) {
+        return validity;
+    }
+
+    // list either up to maxNumKeysForTooltip keys or (maxNumKeysForTooltip-1) keys followed by "and n more keys"
+    const unsigned int numKeysForTooltip = keys.size() > maxNumKeysForTooltip ? maxNumKeysForTooltip - 1 : keys.size();
+
+    QStringList result;
+    result.reserve(3 + 2 + numKeysForTooltip + 2);
+    if (!validity.isEmpty()) {
+        result.push_back(QStringLiteral("<p>"));
+        result.push_back(validity.toHtmlEscaped());
+        result.push_back(QStringLiteral("</p>"));
+    }
+
+    result.push_back(QStringLiteral("<p>"));
+    result.push_back(i18n("Keys:"));
+    {
+        auto it = keys.cbegin();
+        for (unsigned int i = 0; i < numKeysForTooltip; ++i, ++it) {
+            result.push_back(QLatin1String("<br>") + Formatting::summaryLine(*it).toHtmlEscaped());
+        }
+    }
+    if (keys.size() > numKeysForTooltip) {
+        result.push_back(QLatin1String("<br>") + i18ncp("this follows a list of keys", "and 1 more key", "and %1 more keys", keys.size() - numKeysForTooltip));
+    }
+    result.push_back(QStringLiteral("</p>"));
+
+    return result.join(QLatin1Char('\n'));
+}
+
 //
 // Creation and Expiration
 //
@@ -474,7 +535,7 @@ namespace
 static QDate time_t2date(time_t t)
 {
     if (!t) {
-        return QDate();
+        return {};
     }
     QDateTime dt;
     dt.setTime_t(t);
@@ -574,7 +635,7 @@ QDate Formatting::creationDate(const UserID::Signature &sig)
 QString Formatting::displayName(Protocol p)
 {
     if (p == CMS) {
-        return i18nc("X.509/CMS encryption standard", "X.509");
+        return i18nc("X.509/CMS encryption standard", "S/MIME");
     }
     if (p == OpenPGP) {
         return i18n("OpenPGP");
@@ -592,6 +653,12 @@ QString Formatting::type(const Subkey &subkey)
     return QString::fromUtf8(subkey.publicKeyAlgorithmAsString());
 }
 
+QString Formatting::type(const KeyGroup &group)
+{
+    Q_UNUSED(group)
+    return i18nc("a group of keys/certificates", "Group");
+}
+
 //
 // Status / Validity
 //
@@ -790,35 +857,35 @@ QString Formatting::signatureToString(const Signature &sig, const Key &key)
     const bool red   = (sig.summary() & Signature::Red);
     const bool valid = (sig.summary() & Signature::Valid);
 
-    if (red)
-        if (key.isNull())
+    if (red) {
+        if (key.isNull()) {
             if (const char *fpr = sig.fingerprint()) {
                 return i18n("Bad signature by unknown certificate %1: %2", QString::fromLatin1(fpr), QString::fromLocal8Bit(sig.status().asString()));
             } else {
                 return i18n("Bad signature by an unknown certificate: %1", QString::fromLocal8Bit(sig.status().asString()));
             }
-        else {
+        } else {
             return i18n("Bad signature by %1: %2", keyToString(key), QString::fromLocal8Bit(sig.status().asString()));
         }
 
-    else if (valid)
-        if (key.isNull())
+    } else if (valid) {
+        if (key.isNull()) {
             if (const char *fpr = sig.fingerprint()) {
                 return i18n("Good signature by unknown certificate %1.", QString::fromLatin1(fpr));
             } else {
                 return i18n("Good signature by an unknown certificate.");
             }
-        else {
+        } else {
             return i18n("Good signature by %1.", keyToString(key));
         }
 
-    else if (key.isNull())
+    } else if (key.isNull()) {
         if (const char *fpr = sig.fingerprint()) {
             return i18n("Invalid signature by unknown certificate %1: %2", QString::fromLatin1(fpr), QString::fromLocal8Bit(sig.status().asString()));
         } else {
             return i18n("Invalid signature by an unknown certificate: %1", QString::fromLocal8Bit(sig.status().asString()));
         }
-    else {
+    } else {
         return i18n("Invalid signature by %1: %2", keyToString(key), QString::fromLocal8Bit(sig.status().asString()));
     }
 }
@@ -832,10 +899,11 @@ QString Formatting::importMetaData(const Import &import, const QStringList &ids)
     const QString result = importMetaData(import);
     if (result.isEmpty()) {
         return QString();
-    } else
+    } else {
         return result + QLatin1Char('\n') +
                i18n("This certificate was imported from the following sources:") + QLatin1Char('\n') +
                ids.join(QLatin1Char('\n'));
+    }
 }
 
 QString Formatting::importMetaData(const Import &import)
@@ -848,15 +916,17 @@ QString Formatting::importMetaData(const Import &import)
     if (import.error().isCanceled()) {
         return i18n("The import of this certificate was canceled.");
     }
-    if (import.error())
+    if (import.error()) {
         return i18n("An error occurred importing this certificate: %1",
                     QString::fromLocal8Bit(import.error().asString()));
+    }
 
     const unsigned int status = import.status();
-    if (status & Import::NewKey)
+    if (status & Import::NewKey) {
         return (status & Import::ContainedSecretKey)
                ? i18n("This certificate was new to your keystore. The secret key is available.")
                : i18n("This certificate is new to your keystore.");
+    }
 
     QStringList results;
     if (status & Import::NewUserIDs) {
@@ -911,10 +981,30 @@ QString Formatting::summaryLine(const Key &key)
                  Formatting::creationDateString(key));
 }
 
-// Icon for certificate selection indication
-QIcon Formatting::iconForUid(const UserID &uid)
+QString Formatting::summaryLine(const KeyGroup &group)
 {
-    switch (uid.validity()) {
+    switch (group.source()) {
+        case KeyGroup::ApplicationConfig:
+        case KeyGroup::GnuPGConfig:
+            return i18ncp("name of group of keys (n key(s), validity)",
+                          "%2 (1 key, %3)", "%2 (%1 keys, %3)",
+                          group.keys().size(), group.name(), Formatting::complianceStringShort(group));
+        case KeyGroup::Tags:
+            return i18ncp("name of group of keys (n key(s), validity, tag)",
+                          "%2 (1 key, %3, tag)", "%2 (%1 keys, %3, tag)",
+                          group.keys().size(), group.name(), Formatting::complianceStringShort(group));
+        default:
+            return i18ncp("name of group of keys (n key(s), validity, group ...)",
+                          "%2 (1 key, %3, unknown origin)", "%2 (%1 keys, %3, unknown origin)",
+                          group.keys().size(), group.name(), Formatting::complianceStringShort(group));
+    }
+}
+
+namespace
+{
+QIcon iconForValidity(UserID::Validity validity)
+{
+    switch (validity) {
         case UserID::Ultimate:
         case UserID::Full:
         case UserID::Marginal:
@@ -927,6 +1017,13 @@ QIcon Formatting::iconForUid(const UserID &uid)
             return QIcon::fromTheme(QStringLiteral("emblem-information"));
     }
 }
+}
+
+// Icon for certificate selection indication
+QIcon Formatting::iconForUid(const UserID &uid)
+{
+    return iconForValidity(uid.validity());
+}
 
 QString Formatting::validity(const UserID &uid)
 {
@@ -946,37 +1043,58 @@ QString Formatting::validity(const UserID &uid)
     }
 }
 
-bool Formatting::uidsHaveFullValidity(const GpgME::Key &key)
+QString Formatting::validity(const KeyGroup &group)
 {
-    bool oneValid = false;
-    for (const auto &uid: key.userIDs()) {
-        if (uid.isRevoked()) {
-            /* Skip revoked uids */
-            continue;
-        }
-        if (uid.validity() < UserID::Validity::Full) {
-            return false;
-        }
-        /* Only return true if we have found at least one
-         * valid uid. E.g. if all uids are revoked we do
-         * not want to return true here. */
-        oneValid = true;
+    if (group.isNull()) {
+        return QString();
     }
-    return oneValid;
+
+    const KeyGroup::Keys &keys = group.keys();
+    if (keys.size() == 0) {
+        return i18n("This group does not contain any keys.");
+    }
+
+    return getValidityStatement(keys);
+}
+
+namespace
+{
+UserID::Validity minimalValidityOfNotRevokedUserIDs(const Key &key)
+{
+    std::vector<UserID> userIDs = key.userIDs();
+    const auto endOfNotRevokedUserIDs = std::remove_if(userIDs.begin(), userIDs.end(), std::mem_fn(&UserID::isRevoked));
+    const int minValidity = std::accumulate(userIDs.begin(), endOfNotRevokedUserIDs, UserID::Ultimate + 1,
+                                            [] (int validity, const UserID &userID) {
+                                                return std::min(validity, static_cast<int>(userID.validity()));
+                                            });
+    return minValidity <= UserID::Ultimate ? static_cast<UserID::Validity>(minValidity) : UserID::Unknown;
+}
+
+template <typename Container>
+UserID::Validity minimalValidity(const Container& keys)
+{
+    const int minValidity = std::accumulate(keys.cbegin(), keys.cend(), UserID::Ultimate + 1,
+                                            [] (int validity, const Key &key) {
+                                                return std::min<int>(validity, minimalValidityOfNotRevokedUserIDs(key));
+                                            });
+    return minValidity <= UserID::Ultimate ? static_cast<UserID::Validity>(minValidity) : UserID::Unknown;
+}
+}
+
+QIcon Formatting::validityIcon(const KeyGroup &group)
+{
+    return iconForValidity(minimalValidity(group.keys()));
+}
+
+bool Formatting::uidsHaveFullValidity(const Key &key)
+{
+    return minimalValidityOfNotRevokedUserIDs(key) >= UserID::Full;
 }
 
 QString Formatting::complianceMode()
 {
-    const QGpgME::CryptoConfig *const config = QGpgME::cryptoConfig();
-    if (!config) {
-        return QString();
-    }
-    const QGpgME::CryptoConfigEntry *const entry = config->entry(QStringLiteral("gpg"), QStringLiteral("Configuration"), QStringLiteral("compliance"));
-
-    if (!entry || entry->stringValue() == QLatin1String("gnupg")) {
-        return QString();
-    }
-    return entry->stringValue();
+    const auto complianceValue = getCryptoConfigStringValue("gpg", "compliance");
+    return complianceValue == QLatin1String("gnupg") ? QString() : complianceValue;
 }
 
 bool Formatting::isKeyDeVs(const GpgME::Key &key)
@@ -997,13 +1115,13 @@ QString Formatting::complianceStringForKey(const GpgME::Key &key)
 {
     // There will likely be more in the future for other institutions
     // for now we only have DE-VS
-    if (complianceMode() == QLatin1String("de-vs")) {
+    if (Kleo::gnupgIsDeVsCompliant()) {
         if (uidsHaveFullValidity(key) && isKeyDeVs(key)) {
-            return i18nc("VS-NfD conforming is a German standard for restricted documents. For which special restrictions about algorithms apply. The string describes if a key is compliant with that..",
-                         "May be used for VS-NfD-compliant communication.");
+            return i18nc("%1 is a placeholder for the name of a compliance mode. E.g. NATO RESTRICTED compliant or VS-NfD compliant",
+                         "May be used for %1 communication.", deVsString());
         } else {
             return i18nc("VS-NfD-conforming is a German standard for restricted documents. For which special restrictions about algorithms apply. The string describes if a key is compliant to that..",
-                         "May <b>not</b> be used for VS-NfD-compliant communication.");
+                         "May <b>not</b> be used for %1 communication.", deVsString());
         }
     }
     return QString();
@@ -1011,12 +1129,10 @@ QString Formatting::complianceStringForKey(const GpgME::Key &key)
 
 QString Formatting::complianceStringShort(const GpgME::Key &key)
 {
-    if (Formatting::uidsHaveFullValidity(key)) {
-        if (complianceMode() == QLatin1String("de-vs")
-            && Formatting::isKeyDeVs(key)) {
-            return QStringLiteral("★ ") +
-                i18nc("VS-NfD-conforming is a German standard for restricted documents for which special restrictions about algorithms apply.  The string states that a key is compliant with that.",
-                      "VS-NfD-compliant");
+    const bool keyValidityChecked = (key.keyListMode() & GpgME::Validate);
+    if (keyValidityChecked && Formatting::uidsHaveFullValidity(key)) {
+        if (Kleo::gnupgIsDeVsCompliant() && Formatting::isKeyDeVs(key)) {
+            return QStringLiteral("★ ") + deVsString(true);
         }
         return i18nc("As in all user IDs are valid.", "certified");
     }
@@ -1032,8 +1148,23 @@ QString Formatting::complianceStringShort(const GpgME::Key &key)
     if (key.isInvalid()) {
         return i18n("invalid");
     }
+    if (keyValidityChecked) {
+        return i18nc("As in not all user IDs are valid.", "not certified");
+    }
 
-    return i18nc("As in not all user IDs are valid.", "not certified");
+    return i18nc("The validity of the user IDs has not been/could not be checked", "not checked");
+}
+
+QString Formatting::complianceStringShort(const KeyGroup &group)
+{
+    const KeyGroup::Keys &keys = group.keys();
+
+    const bool allKeysFullyValid = std::all_of(keys.cbegin(), keys.cend(), &Formatting::uidsHaveFullValidity);
+    if (allKeysFullyValid) {
+        return i18nc("As in all keys are valid.", "all certified");
+    }
+
+    return i18nc("As in not all keys are valid.", "not all certified");
 }
 
 QString Formatting::prettyID(const char *id)
@@ -1072,3 +1203,58 @@ QString Formatting::origin(int o)
           return i18n("Unknown");
     }
 }
+
+QString Formatting::deVsString(bool compliant)
+{
+    const auto filter = KeyFilterManager::instance()->keyFilterByID(compliant ?
+            QStringLiteral("de-vs-filter") :
+            QStringLiteral("not-de-vs-filter"));
+    if (!filter) {
+        return compliant ? i18n("VS-NfD compliant") : i18n("Not VS-NfD compliant");
+    }
+    return filter->name();
+}
+
+namespace
+{
+QString formatTrustScope(const char *trustScope)
+{
+    static const QRegularExpression escapedNonAlphaNum{QStringLiteral(R"(\\([^0-9A-Za-z]))")};
+
+    const auto scopeRegExp = QString::fromUtf8(trustScope);
+    if (scopeRegExp.startsWith(u"<[^>]+[@.]") && scopeRegExp.endsWith(u">$")) {
+        // looks like a trust scope regular expression created by gpg
+        auto domain = scopeRegExp.mid(10, scopeRegExp.size() - 10 - 2);
+        domain.replace(escapedNonAlphaNum, QStringLiteral(R"(\1)"));
+        return domain;
+    }
+    return scopeRegExp;
+}
+}
+
+QString Formatting::trustSignatureDomain(const GpgME::UserID::Signature &sig)
+{
+#ifdef GPGMEPP_SUPPORTS_TRUST_SIGNATURES
+    return formatTrustScope(sig.trustScope());
+#else
+    return {};
+#endif
+}
+
+QString Formatting::trustSignature(const GpgME::UserID::Signature &sig)
+{
+#ifdef GPGMEPP_SUPPORTS_TRUST_SIGNATURES
+    switch (sig.trustValue()) {
+    case TrustSignatureTrust::Partial:
+        return i18nc("Certifies this key as partially trusted introducer for 'domain name'.",
+                     "Certifies this key as partially trusted introducer for '%1'.", trustSignatureDomain(sig));
+    case TrustSignatureTrust::Complete:
+        return i18nc("Certifies this key as fully trusted introducer for 'domain name'.",
+                     "Certifies this key as fully trusted introducer for '%1'.", trustSignatureDomain(sig));
+    default:
+        return {};
+    }
+#else
+    return {};
+#endif
+}
