@@ -2,43 +2,21 @@
     utils/formatting.h
 
     This file is part of Kleopatra, the KDE keymanager
-    Copyright (c) 2007 Klarälvdalens Datakonsult AB
+    SPDX-FileCopyrightText: 2007 Klarälvdalens Datakonsult AB
+    SPDX-FileCopyrightText: 2021 g10 Code GmbH
+    SPDX-FileContributor: Ingo Klöcker <dev@ingo-kloecker.de>
 
-    Kleopatra is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    Kleopatra is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
-
-    In addition, as a special exception, the copyright holders give
-    permission to link the code of this program with any edition of
-    the Qt library by Trolltech AS, Norway (or with modified versions
-    of Qt that use the same license as Qt), and distribute linked
-    combinations including the two.  You must obey the GNU General
-    Public License in all respects for all of the code used other than
-    Qt.  If you modify this file, you may extend this exception to
-    your version of the file, but you are not obligated to do so.  If
-    you do not wish to do so, delete this exception statement from
-    your version.
+    SPDX-License-Identifier: GPL-2.0-or-later
 */
 
-#ifndef __KLEOPATRA_UTILS_FORMATTING_H__
-#define __KLEOPATRA_UTILS_FORMATTING_H__
+#pragma once
 
 #include <gpgme++/key.h>
 
-#include <kleo_export.h>
+#include "kleo_export.h"
 
 class QString;
-class QStringList;
+#include <QStringList>
 class QDate;
 class QIcon;
 
@@ -49,6 +27,8 @@ class Import;
 
 namespace Kleo
 {
+class KeyGroup;
+
 namespace Formatting
 {
 
@@ -92,6 +72,7 @@ enum ToolTipOption {
 };
 
 KLEO_EXPORT QString toolTip(const GpgME::Key &key, int opts);
+KLEO_EXPORT QString toolTip(const Kleo::KeyGroup &group, int opts);
 
 KLEO_EXPORT QString expirationDateString(const GpgME::Key &key);
 KLEO_EXPORT QString expirationDateString(const GpgME::Subkey &subkey);
@@ -113,6 +94,7 @@ KLEO_EXPORT QString dateString(time_t t);
 KLEO_EXPORT QString displayName(GpgME::Protocol prot);
 KLEO_EXPORT QString type(const GpgME::Key &key);
 KLEO_EXPORT QString type(const GpgME::Subkey &subkey);
+KLEO_EXPORT QString type(const Kleo::KeyGroup &group);
 
 KLEO_EXPORT QString ownerTrustShort(const GpgME::Key &key);
 KLEO_EXPORT QString ownerTrustShort(GpgME::Key::OwnerTrust trust);
@@ -123,6 +105,8 @@ KLEO_EXPORT QString validityShort(const GpgME::UserID::Signature &sig);
 KLEO_EXPORT QIcon   validityIcon(const GpgME::UserID::Signature &sig);
 /* A sentence about the validity of the UserID */
 KLEO_EXPORT QString validity(const GpgME::UserID &uid);
+KLEO_EXPORT QString validity(const Kleo::KeyGroup &group);
+KLEO_EXPORT QIcon validityIcon(const Kleo::KeyGroup &group);
 
 KLEO_EXPORT QString formatForComboBox(const GpgME::Key &key);
 
@@ -138,6 +122,7 @@ KLEO_EXPORT QString importMetaData(const GpgME::Import &import, const QStringLis
 KLEO_EXPORT QString formatOverview(const GpgME::Key &key);
 KLEO_EXPORT QString usageString(const GpgME::Subkey &subkey);
 KLEO_EXPORT QString summaryLine(const GpgME::Key &key);
+KLEO_EXPORT QString summaryLine(const KeyGroup &group);
 
 KLEO_EXPORT QIcon iconForUid(const GpgME::UserID &uid);
 
@@ -145,11 +130,21 @@ KLEO_EXPORT QIcon iconForUid(const GpgME::UserID &uid);
 KLEO_EXPORT bool uidsHaveFullValidity(const GpgME::Key &key);
 
 /* The compliance mode of the gnupg system. Empty if compliance
- * mode is not set. */
-KLEO_EXPORT QString complianceMode();
+ * mode is not set.
+ * Use Kleo::gnupgComplianceMode() instead.
+ */
+KLEO_DEPRECATED_EXPORT QString complianceMode();
 
 /* Is the given key in compliance with CO_DE_VS?  */
 KLEO_EXPORT bool isKeyDeVs(const GpgME::Key &key);
+/* Localized string describing the name of the VS-NfD Compliance filter. If
+ * compliant is false the name of the not Compliant filter.
+ *
+ * This is required to make the string configurable which is
+ * a common request from users because VS-NfD compliance is called
+ * differently in different environments. E.g NATO RESTRICTED or
+ * EU RESTRICTED. */
+KLEO_EXPORT QString deVsString (bool compliant = true);
 
 /* A sentence if the key confirms to the current compliance mode */
 KLEO_EXPORT QString complianceStringForKey(const GpgME::Key &key);
@@ -158,10 +153,15 @@ KLEO_EXPORT QString complianceStringForKey(const GpgME::Key &key);
  * given key, including any conformance statements relevant to the
  * current conformance mode.  */
 KLEO_EXPORT QString complianceStringShort(const GpgME::Key &key);
+KLEO_EXPORT QString complianceStringShort(const Kleo::KeyGroup &group);
 
 /* The origin of the key mapped to a localized string */
 KLEO_EXPORT QString origin(int o);
+
+/* Human-readable trust signature scope (for trust signature regexp created by GnuPG) */
+KLEO_EXPORT QString trustSignatureDomain(const GpgME::UserID::Signature &sig);
+/* Summary of trust signature properties */
+KLEO_EXPORT QString trustSignature(const GpgME::UserID::Signature &sig);
 }
 }
 
-#endif /* __KLEOPATRA_UTILS_FORMATTING_H__ */
